searchData={"items":[{"type":"module","title":"Teiserver","doc":"Teiserver is a middleware server library; designed for usage with games. It handles game-agnostic issues (chat, searching for games, match history) and allows you to implement the game-specific items you care about.\n\nIn a peer-to-peer setting each client will communicate to each other client as and when they need to. With a middleware server every client communicates via the middleware server and the server acts as a single source of truth.\n\n```mermaid\ngraph TD;\n  srv{{Middleware Server}};\n  srv <--> User1;\n  srv <--> User2;\n  srv <--> User3;\n  srv <--> User4;\n  Host1 <--> srv;\n  Host2 <--> srv;\n  Bot1 <--> srv;\n  Bot2 <--> srv;\n```\n\nTo use Teiserver you write an endpoint which your game connects to over a network. The endpoint handles the messages from client apps and makes calls to Teiserver; acting as wrapper around the Teiserver API.","ref":"Teiserver.html"},{"type":"module","title":"Main guides: - Teiserver","doc":"- [Installation](guides/installation.md)\n- [Hello world](guides/hello_world.md)\n- [Program structure](guides/program_structure.md)\n- [Snippets](guides/snippets.md)","ref":"Teiserver.html#module-main-guides"},{"type":"module","title":"Contexts - Teiserver","doc":"These are the main modules you will be interacting with in Teiserver. They typically delegate all their functions to something more specific but the context module will be the preferred point of contact with the Teiserver library.\n\nTeiserver has some other publicly accessible functions for those who want to write more advanced or complex functionality but in theory everything you need should be accessible from the relevant context.\n\n#","ref":"Teiserver.html#module-contexts"},{"type":"module","title":"Context overview - Teiserver","doc":"- **Accounts**: Users\n- **Communication**: Chat\n- **Community**: Social interactions between players\n- **Connections**: User activity\n- **Game**: Game/playing functionality\n- **Logging**: Logging of events and numbers\n- **Moderation**: Handling disruptive users\n- **Settings**: Key-Value pairs for users and the system\n- **Telemetry**: Moment to moment events","ref":"Teiserver.html#module-context-overview"},{"type":"function","title":"Teiserver.deterministic_uuid/1","doc":"","ref":"Teiserver.html#deterministic_uuid/1"},{"type":"function","title":"Teiserver.uuid/0","doc":"","ref":"Teiserver.html#uuid/0"},{"type":"type","title":"Teiserver.lobby_id/0","doc":"","ref":"Teiserver.html#t:lobby_id/0"},{"type":"type","title":"Teiserver.match_id/0","doc":"","ref":"Teiserver.html#t:match_id/0"},{"type":"type","title":"Teiserver.party_id/0","doc":"","ref":"Teiserver.html#t:party_id/0"},{"type":"type","title":"Teiserver.query_args/0","doc":"","ref":"Teiserver.html#t:query_args/0"},{"type":"type","title":"Teiserver.queue_id/0","doc":"","ref":"Teiserver.html#t:queue_id/0"},{"type":"type","title":"Teiserver.seconds/0","doc":"","ref":"Teiserver.html#t:seconds/0"},{"type":"type","title":"Teiserver.team_number/0","doc":"","ref":"Teiserver.html#t:team_number/0"},{"type":"type","title":"Teiserver.user_id/0","doc":"","ref":"Teiserver.html#t:user_id/0"},{"type":"module","title":"Teiserver.Api","doc":"A set of functions for a basic usage of Teiserver. The purpose is\nto allow you to start with importing only this module and then\nimport others as your needs grow more complex.","ref":"Teiserver.Api.html"},{"type":"function","title":"Teiserver.Api.add_client_to_lobby/2","doc":"","ref":"Teiserver.Api.html#add_client_to_lobby/2"},{"type":"function","title":"Teiserver.Api.can_add_client_to_lobby/2","doc":"","ref":"Teiserver.Api.html#can_add_client_to_lobby/2"},{"type":"function","title":"Teiserver.Api.can_add_client_to_lobby/3","doc":"","ref":"Teiserver.Api.html#can_add_client_to_lobby/3"},{"type":"function","title":"Teiserver.Api.close_lobby/1","doc":"","ref":"Teiserver.Api.html#close_lobby/1"},{"type":"function","title":"Teiserver.Api.connect_user/1","doc":"Makes use of `Teiserver.Connections.ClientLib.connect_user/1` to connect\nand then also subscribes you to the following pubsubs:\n- [Teiserver.Connections.Client](documentation/pubsubs/client.md#teiserver-connections-client-user_id)\n- [Teiserver.Communication.User](documentation/pubsubs/communication.md#teiserver-communication-user-user_id)\n\nAlways returns `:ok`","ref":"Teiserver.Api.html#connect_user/1"},{"type":"function","title":"Teiserver.Api.cycle_lobby/1","doc":"","ref":"Teiserver.Api.html#cycle_lobby/1"},{"type":"function","title":"Teiserver.Api.end_match/2","doc":"","ref":"Teiserver.Api.html#end_match/2"},{"type":"function","title":"Teiserver.Api.get_client/1","doc":"","ref":"Teiserver.Api.html#get_client/1"},{"type":"function","title":"Teiserver.Api.get_lobby/1","doc":"","ref":"Teiserver.Api.html#get_lobby/1"},{"type":"function","title":"Teiserver.Api.get_lobby_summary/1","doc":"","ref":"Teiserver.Api.html#get_lobby_summary/1"},{"type":"function","title":"Teiserver.Api.get_room_by_name_or_id/1","doc":"","ref":"Teiserver.Api.html#get_room_by_name_or_id/1"},{"type":"function","title":"Teiserver.Api.get_user_by_id/1","doc":"","ref":"Teiserver.Api.html#get_user_by_id/1"},{"type":"function","title":"Teiserver.Api.get_user_by_name/1","doc":"","ref":"Teiserver.Api.html#get_user_by_name/1"},{"type":"function","title":"Teiserver.Api.list_lobby_ids/0","doc":"","ref":"Teiserver.Api.html#list_lobby_ids/0"},{"type":"function","title":"Teiserver.Api.list_recent_room_messages/1","doc":"","ref":"Teiserver.Api.html#list_recent_room_messages/1"},{"type":"function","title":"Teiserver.Api.lobby_exists?/1","doc":"","ref":"Teiserver.Api.html#lobby_exists?/1"},{"type":"function","title":"Teiserver.Api.maybe_authenticate_user/2","doc":"Takes a name and password, tries to authenticate the user.","ref":"Teiserver.Api.html#maybe_authenticate_user/2"},{"type":"function","title":"Examples - Teiserver.Api.maybe_authenticate_user/2","doc":"iex> maybe_authenticate_user(\"Alice\", \"password1\")\n    {:ok, %User{}}\n\n    iex> maybe_authenticate_user(\"Bob\", \"bad password\")\n    {:error, :bad_password}\n\n    iex> maybe_authenticate_user(\"Chris\", \"password1\")\n    {:error, :no_user}","ref":"Teiserver.Api.html#maybe_authenticate_user/2-examples"},{"type":"function","title":"Teiserver.Api.open_lobby/2","doc":"","ref":"Teiserver.Api.html#open_lobby/2"},{"type":"function","title":"Teiserver.Api.register_user/3","doc":"Takes a name, email and password. Creates a user with them.","ref":"Teiserver.Api.html#register_user/3"},{"type":"function","title":"Examples - Teiserver.Api.register_user/3","doc":"iex> register_user(\"Alice\", \"alice@alice\", \"password1\")\n    {:ok, %User{}}\n\n    iex> register_user(\"Bob\", \"bob@bob\", \"1\")\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Api.html#register_user/3-examples"},{"type":"function","title":"Teiserver.Api.remove_client_from_lobby/2","doc":"","ref":"Teiserver.Api.html#remove_client_from_lobby/2"},{"type":"function","title":"Teiserver.Api.send_direct_message/3","doc":"","ref":"Teiserver.Api.html#send_direct_message/3"},{"type":"function","title":"Teiserver.Api.send_lobby_message/3","doc":"","ref":"Teiserver.Api.html#send_lobby_message/3"},{"type":"function","title":"Teiserver.Api.send_match_message/3","doc":"","ref":"Teiserver.Api.html#send_match_message/3"},{"type":"function","title":"Teiserver.Api.send_room_message/3","doc":"","ref":"Teiserver.Api.html#send_room_message/3"},{"type":"function","title":"Teiserver.Api.start_match/1","doc":"","ref":"Teiserver.Api.html#start_match/1"},{"type":"function","title":"Teiserver.Api.stream_lobby_summaries/0","doc":"","ref":"Teiserver.Api.html#stream_lobby_summaries/0"},{"type":"function","title":"Teiserver.Api.stream_lobby_summaries/1","doc":"","ref":"Teiserver.Api.html#stream_lobby_summaries/1"},{"type":"function","title":"Teiserver.Api.subscribe_to_lobby/1","doc":"","ref":"Teiserver.Api.html#subscribe_to_lobby/1"},{"type":"function","title":"Teiserver.Api.subscribe_to_match_messages/1","doc":"","ref":"Teiserver.Api.html#subscribe_to_match_messages/1"},{"type":"function","title":"Teiserver.Api.subscribe_to_room_messages/1","doc":"","ref":"Teiserver.Api.html#subscribe_to_room_messages/1"},{"type":"function","title":"Teiserver.Api.subscribe_to_user_messaging/1","doc":"","ref":"Teiserver.Api.html#subscribe_to_user_messaging/1"},{"type":"function","title":"Teiserver.Api.unsubscribe_from_lobby/1","doc":"","ref":"Teiserver.Api.html#unsubscribe_from_lobby/1"},{"type":"function","title":"Teiserver.Api.unsubscribe_from_match_messages/1","doc":"","ref":"Teiserver.Api.html#unsubscribe_from_match_messages/1"},{"type":"function","title":"Teiserver.Api.unsubscribe_from_room_messages/1","doc":"","ref":"Teiserver.Api.html#unsubscribe_from_room_messages/1"},{"type":"function","title":"Teiserver.Api.unsubscribe_from_user_messaging/1","doc":"","ref":"Teiserver.Api.html#unsubscribe_from_user_messaging/1"},{"type":"function","title":"Teiserver.Api.update_client/3","doc":"","ref":"Teiserver.Api.html#update_client/3"},{"type":"function","title":"Teiserver.Api.update_client_full/3","doc":"","ref":"Teiserver.Api.html#update_client_full/3"},{"type":"function","title":"Teiserver.Api.update_client_in_lobby/3","doc":"","ref":"Teiserver.Api.html#update_client_in_lobby/3"},{"type":"function","title":"Teiserver.Api.update_lobby/2","doc":"","ref":"Teiserver.Api.html#update_lobby/2"},{"type":"module","title":"Teiserver.System","doc":"","ref":"Teiserver.System.html"},{"type":"module","title":"Teiserver.System.ClusterManager","doc":"The cluster manager for handling adding nodes to a cluster.","ref":"Teiserver.System.ClusterManager.html"},{"type":"function","title":"Teiserver.System.ClusterManager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Teiserver.System.ClusterManager.html#child_spec/1"},{"type":"function","title":"Teiserver.System.ClusterManager.start_link/2","doc":"","ref":"Teiserver.System.ClusterManager.html#start_link/2"},{"type":"module","title":"Teiserver.System.ClusterManagerSupervisor","doc":"The dynamic supervisor for the ClusterManager","ref":"Teiserver.System.ClusterManagerSupervisor.html"},{"type":"function","title":"Teiserver.System.ClusterManagerSupervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Teiserver.System.ClusterManagerSupervisor.html#child_spec/1"},{"type":"function","title":"Teiserver.System.ClusterManagerSupervisor.start_cluster_manager_supervisor_children/0","doc":"","ref":"Teiserver.System.ClusterManagerSupervisor.html#start_cluster_manager_supervisor_children/0"},{"type":"function","title":"Teiserver.System.ClusterManagerSupervisor.start_link/1","doc":"","ref":"Teiserver.System.ClusterManagerSupervisor.html#start_link/1"},{"type":"module","title":"Teiserver.System.ClusterMember","doc":"# Match\nA clustering method making use of a database to setup and update the cluster.\n\n#","ref":"Teiserver.System.ClusterMember.html"},{"type":"module","title":"Attributes - Teiserver.System.ClusterMember","doc":"* `:host` - The host name, e.g. \"server1.host.co.uk\"","ref":"Teiserver.System.ClusterMember.html#module-attributes"},{"type":"function","title":"Teiserver.System.ClusterMember.changeset/2","doc":"Builds a changeset based on the `struct` and `params`.","ref":"Teiserver.System.ClusterMember.html#changeset/2"},{"type":"type","title":"Teiserver.System.ClusterMember.id/0","doc":"","ref":"Teiserver.System.ClusterMember.html#t:id/0"},{"type":"type","title":"Teiserver.System.ClusterMember.t/0","doc":"","ref":"Teiserver.System.ClusterMember.html#t:t/0"},{"type":"module","title":"Teiserver.System.ClusterMemberLib","doc":"Library of cluster_member related functions.","ref":"Teiserver.System.ClusterMemberLib.html"},{"type":"function","title":"Teiserver.System.ClusterMemberLib.change_cluster_member/2","doc":"Returns an `%Ecto.Changeset{}` for tracking cluster_member changes.","ref":"Teiserver.System.ClusterMemberLib.html#change_cluster_member/2"},{"type":"function","title":"Examples - Teiserver.System.ClusterMemberLib.change_cluster_member/2","doc":"iex> change_cluster_member(cluster_member)\n    %Ecto.Changeset{data: %ClusterMember{}}","ref":"Teiserver.System.ClusterMemberLib.html#change_cluster_member/2-examples"},{"type":"function","title":"Teiserver.System.ClusterMemberLib.create_cluster_member/1","doc":"Creates a cluster_member.","ref":"Teiserver.System.ClusterMemberLib.html#create_cluster_member/1"},{"type":"function","title":"Examples - Teiserver.System.ClusterMemberLib.create_cluster_member/1","doc":"iex> create_cluster_member(%{field: value})\n    {:ok, %ClusterMember{}}\n\n    iex> create_cluster_member(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.System.ClusterMemberLib.html#create_cluster_member/1-examples"},{"type":"function","title":"Teiserver.System.ClusterMemberLib.delete_cluster_member/1","doc":"Deletes a cluster_member.","ref":"Teiserver.System.ClusterMemberLib.html#delete_cluster_member/1"},{"type":"function","title":"Examples - Teiserver.System.ClusterMemberLib.delete_cluster_member/1","doc":"iex> delete_cluster_member(cluster_member)\n    {:ok, %ClusterMember{}}\n\n    iex> delete_cluster_member(cluster_member)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.System.ClusterMemberLib.html#delete_cluster_member/1-examples"},{"type":"function","title":"Teiserver.System.ClusterMemberLib.get_cluster_member/2","doc":"Gets a single cluster_member.\n\nReturns nil if the ClusterMember does not exist.","ref":"Teiserver.System.ClusterMemberLib.html#get_cluster_member/2"},{"type":"function","title":"Examples - Teiserver.System.ClusterMemberLib.get_cluster_member/2","doc":"iex> get_cluster_member(123)\n    %ClusterMember{}\n\n    iex> get_cluster_member(456)\n    nil","ref":"Teiserver.System.ClusterMemberLib.html#get_cluster_member/2-examples"},{"type":"function","title":"Teiserver.System.ClusterMemberLib.get_cluster_member!/2","doc":"Gets a single cluster_member.\n\nRaises `Ecto.NoResultsError` if the ClusterMember does not exist.","ref":"Teiserver.System.ClusterMemberLib.html#get_cluster_member!/2"},{"type":"function","title":"Examples - Teiserver.System.ClusterMemberLib.get_cluster_member!/2","doc":"iex> get_cluster_member!(123)\n    %ClusterMember{}\n\n    iex> get_cluster_member!(456)\n    ** (Ecto.NoResultsError)","ref":"Teiserver.System.ClusterMemberLib.html#get_cluster_member!/2-examples"},{"type":"function","title":"Teiserver.System.ClusterMemberLib.list_cluster_members/1","doc":"Returns the list of cluster_members.","ref":"Teiserver.System.ClusterMemberLib.html#list_cluster_members/1"},{"type":"function","title":"Examples - Teiserver.System.ClusterMemberLib.list_cluster_members/1","doc":"iex> list_cluster_members()\n    [%ClusterMember{}, ...]","ref":"Teiserver.System.ClusterMemberLib.html#list_cluster_members/1-examples"},{"type":"function","title":"Teiserver.System.ClusterMemberLib.update_cluster_member/2","doc":"Updates a cluster_member.","ref":"Teiserver.System.ClusterMemberLib.html#update_cluster_member/2"},{"type":"function","title":"Examples - Teiserver.System.ClusterMemberLib.update_cluster_member/2","doc":"iex> update_cluster_member(cluster_member, %{field: new_value})\n    {:ok, %ClusterMember{}}\n\n    iex> update_cluster_member(cluster_member, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.System.ClusterMemberLib.html#update_cluster_member/2-examples"},{"type":"module","title":"Teiserver.Account","doc":"The contextual module for:\n- `Teiserver.Account.User`\n- `Teiserver.Account.ExtraUserData`\n- `Teiserver.Account.Relationship`\n- `Teiserver.Account.Friend`\n- `Teiserver.Account.FriendRequest`\n- `Teiserver.Account.Achievement`\n- `Teiserver.Account.AchievementType`","ref":"Teiserver.Account.html"},{"type":"function","title":"Teiserver.Account.allow?/2","doc":"","ref":"Teiserver.Account.html#allow?/2"},{"type":"function","title":"Teiserver.Account.change_user/2","doc":"","ref":"Teiserver.Account.html#change_user/2"},{"type":"function","title":"Teiserver.Account.create_user/1","doc":"","ref":"Teiserver.Account.html#create_user/1"},{"type":"function","title":"Teiserver.Account.delete_user/1","doc":"","ref":"Teiserver.Account.html#delete_user/1"},{"type":"function","title":"Teiserver.Account.generate_password/0","doc":"","ref":"Teiserver.Account.html#generate_password/0"},{"type":"function","title":"Teiserver.Account.get_user/2","doc":"","ref":"Teiserver.Account.html#get_user/2"},{"type":"function","title":"Teiserver.Account.get_user!/2","doc":"","ref":"Teiserver.Account.html#get_user!/2"},{"type":"function","title":"Teiserver.Account.get_user_by_email/1","doc":"","ref":"Teiserver.Account.html#get_user_by_email/1"},{"type":"function","title":"Teiserver.Account.get_user_by_id/1","doc":"","ref":"Teiserver.Account.html#get_user_by_id/1"},{"type":"function","title":"Teiserver.Account.get_user_by_name/1","doc":"","ref":"Teiserver.Account.html#get_user_by_name/1"},{"type":"function","title":"Teiserver.Account.list_users/1","doc":"","ref":"Teiserver.Account.html#list_users/1"},{"type":"function","title":"Teiserver.Account.register_user/1","doc":"","ref":"Teiserver.Account.html#register_user/1"},{"type":"function","title":"Teiserver.Account.restricted?/2","doc":"","ref":"Teiserver.Account.html#restricted?/2"},{"type":"function","title":"Teiserver.Account.update_limited_user/2","doc":"","ref":"Teiserver.Account.html#update_limited_user/2"},{"type":"function","title":"Teiserver.Account.update_password/2","doc":"","ref":"Teiserver.Account.html#update_password/2"},{"type":"function","title":"Teiserver.Account.update_user/2","doc":"","ref":"Teiserver.Account.html#update_user/2"},{"type":"function","title":"Teiserver.Account.user_name_acceptable?/1","doc":"","ref":"Teiserver.Account.html#user_name_acceptable?/1"},{"type":"function","title":"Teiserver.Account.valid_password?/2","doc":"","ref":"Teiserver.Account.html#valid_password?/2"},{"type":"module","title":"Teiserver.Communication","doc":"The contextual module for:\n- `Teiserver.Communication.Room`\n- `Teiserver.Communication.RoomMessage`\n- `Teiserver.Communication.DirectMessage`\n- `Teiserver.Communication.MatchMessage`\n- `Teiserver.Communication.PartyMessage`","ref":"Teiserver.Communication.html"},{"type":"function","title":"Teiserver.Communication.change_direct_message/2","doc":"","ref":"Teiserver.Communication.html#change_direct_message/2"},{"type":"function","title":"Teiserver.Communication.change_match_message/2","doc":"","ref":"Teiserver.Communication.html#change_match_message/2"},{"type":"function","title":"Teiserver.Communication.change_room/2","doc":"","ref":"Teiserver.Communication.html#change_room/2"},{"type":"function","title":"Teiserver.Communication.change_room_message/2","doc":"","ref":"Teiserver.Communication.html#change_room_message/2"},{"type":"function","title":"Teiserver.Communication.create_direct_message/1","doc":"","ref":"Teiserver.Communication.html#create_direct_message/1"},{"type":"function","title":"Teiserver.Communication.create_match_message/1","doc":"","ref":"Teiserver.Communication.html#create_match_message/1"},{"type":"function","title":"Teiserver.Communication.create_room/1","doc":"","ref":"Teiserver.Communication.html#create_room/1"},{"type":"function","title":"Teiserver.Communication.create_room_message/1","doc":"","ref":"Teiserver.Communication.html#create_room_message/1"},{"type":"function","title":"Teiserver.Communication.delete_direct_message/1","doc":"","ref":"Teiserver.Communication.html#delete_direct_message/1"},{"type":"function","title":"Teiserver.Communication.delete_match_message/1","doc":"","ref":"Teiserver.Communication.html#delete_match_message/1"},{"type":"function","title":"Teiserver.Communication.delete_room/1","doc":"","ref":"Teiserver.Communication.html#delete_room/1"},{"type":"function","title":"Teiserver.Communication.delete_room_message/1","doc":"","ref":"Teiserver.Communication.html#delete_room_message/1"},{"type":"function","title":"Teiserver.Communication.get_direct_message/2","doc":"","ref":"Teiserver.Communication.html#get_direct_message/2"},{"type":"function","title":"Teiserver.Communication.get_direct_message!/2","doc":"","ref":"Teiserver.Communication.html#get_direct_message!/2"},{"type":"function","title":"Teiserver.Communication.get_match_message/2","doc":"","ref":"Teiserver.Communication.html#get_match_message/2"},{"type":"function","title":"Teiserver.Communication.get_match_message!/2","doc":"","ref":"Teiserver.Communication.html#get_match_message!/2"},{"type":"function","title":"Teiserver.Communication.get_or_create_room/1","doc":"","ref":"Teiserver.Communication.html#get_or_create_room/1"},{"type":"function","title":"Teiserver.Communication.get_room/2","doc":"","ref":"Teiserver.Communication.html#get_room/2"},{"type":"function","title":"Teiserver.Communication.get_room!/2","doc":"","ref":"Teiserver.Communication.html#get_room!/2"},{"type":"function","title":"Teiserver.Communication.get_room_by_name_or_id/1","doc":"","ref":"Teiserver.Communication.html#get_room_by_name_or_id/1"},{"type":"function","title":"Teiserver.Communication.get_room_message/2","doc":"","ref":"Teiserver.Communication.html#get_room_message/2"},{"type":"function","title":"Teiserver.Communication.get_room_message!/2","doc":"","ref":"Teiserver.Communication.html#get_room_message!/2"},{"type":"function","title":"Teiserver.Communication.list_direct_messages/1","doc":"","ref":"Teiserver.Communication.html#list_direct_messages/1"},{"type":"function","title":"Teiserver.Communication.list_direct_messages_between_users/3","doc":"","ref":"Teiserver.Communication.html#list_direct_messages_between_users/3"},{"type":"function","title":"Teiserver.Communication.list_direct_messages_for_user/2","doc":"","ref":"Teiserver.Communication.html#list_direct_messages_for_user/2"},{"type":"function","title":"Teiserver.Communication.list_direct_messages_from_user/2","doc":"","ref":"Teiserver.Communication.html#list_direct_messages_from_user/2"},{"type":"function","title":"Teiserver.Communication.list_direct_messages_from_user_to_user/3","doc":"","ref":"Teiserver.Communication.html#list_direct_messages_from_user_to_user/3"},{"type":"function","title":"Teiserver.Communication.list_direct_messages_to_user/2","doc":"","ref":"Teiserver.Communication.html#list_direct_messages_to_user/2"},{"type":"function","title":"Teiserver.Communication.list_match_messages/1","doc":"","ref":"Teiserver.Communication.html#list_match_messages/1"},{"type":"function","title":"Teiserver.Communication.list_recent_match_messages/2","doc":"","ref":"Teiserver.Communication.html#list_recent_match_messages/2"},{"type":"function","title":"Teiserver.Communication.list_recent_room_messages/2","doc":"","ref":"Teiserver.Communication.html#list_recent_room_messages/2"},{"type":"function","title":"Teiserver.Communication.list_room_messages/1","doc":"","ref":"Teiserver.Communication.html#list_room_messages/1"},{"type":"function","title":"Teiserver.Communication.list_rooms/1","doc":"","ref":"Teiserver.Communication.html#list_rooms/1"},{"type":"function","title":"Teiserver.Communication.send_direct_message/4","doc":"","ref":"Teiserver.Communication.html#send_direct_message/4"},{"type":"function","title":"Teiserver.Communication.send_lobby_message/3","doc":"","ref":"Teiserver.Communication.html#send_lobby_message/3"},{"type":"function","title":"Teiserver.Communication.send_match_message/4","doc":"","ref":"Teiserver.Communication.html#send_match_message/4"},{"type":"function","title":"Teiserver.Communication.send_room_message/4","doc":"","ref":"Teiserver.Communication.html#send_room_message/4"},{"type":"function","title":"Teiserver.Communication.subscribe_to_match_messages/1","doc":"","ref":"Teiserver.Communication.html#subscribe_to_match_messages/1"},{"type":"function","title":"Teiserver.Communication.subscribe_to_room_messages/1","doc":"","ref":"Teiserver.Communication.html#subscribe_to_room_messages/1"},{"type":"function","title":"Teiserver.Communication.subscribe_to_user_messaging/1","doc":"","ref":"Teiserver.Communication.html#subscribe_to_user_messaging/1"},{"type":"function","title":"Teiserver.Communication.unsubscribe_from_match_messages/1","doc":"","ref":"Teiserver.Communication.html#unsubscribe_from_match_messages/1"},{"type":"function","title":"Teiserver.Communication.unsubscribe_from_room_messages/1","doc":"","ref":"Teiserver.Communication.html#unsubscribe_from_room_messages/1"},{"type":"function","title":"Teiserver.Communication.unsubscribe_from_user_messaging/1","doc":"","ref":"Teiserver.Communication.html#unsubscribe_from_user_messaging/1"},{"type":"function","title":"Teiserver.Communication.update_direct_message/2","doc":"","ref":"Teiserver.Communication.html#update_direct_message/2"},{"type":"function","title":"Teiserver.Communication.update_match_message/2","doc":"","ref":"Teiserver.Communication.html#update_match_message/2"},{"type":"function","title":"Teiserver.Communication.update_room/2","doc":"","ref":"Teiserver.Communication.html#update_room/2"},{"type":"function","title":"Teiserver.Communication.update_room_message/2","doc":"","ref":"Teiserver.Communication.html#update_room_message/2"},{"type":"module","title":"Teiserver.Community","doc":"The contextual module for:\n- `Teiserver.Community.Party`\n- `Teiserver.Community.Clan`\n- `Teiserver.Community.ClanMembership`\n- `Teiserver.Community.ClanInvite`\n- `Teiserver.Community.Accolade`\n- `Teiserver.Community.AccoladeType`","ref":"Teiserver.Community.html"},{"type":"module","title":"Teiserver.Connections","doc":"The context for all things connection related, mostly `Teiserver.Connections.Client`\n\n# Clients\nClients are represented by a [`process`](Teiserver.Connections.ClientServer.html) holding their state, this process controls all updates to their state.","ref":"Teiserver.Connections.html"},{"type":"module","title":"Creating a client - Teiserver.Connections","doc":"Clients are created using `connect_user/1`, this will create (if it doesn't already exist) a client process. Any process connecting a user will be subscribed to the `Teiserver.ClientUpdates:{user_id}` channel.","ref":"Teiserver.Connections.html#module-creating-a-client"},{"type":"module","title":"Destroying clients - Teiserver.Connections","doc":"When a client loses all connections it will begin a countdown [(which can be configured)](config.html#client_destroy_timeout_seconds-default-300). If a connection is re-established in that time the countdown will be reset. If the countdown reaches zero then the client process will be stopped and the client will no longer show up on any lists.","ref":"Teiserver.Connections.html#module-destroying-clients"},{"type":"function","title":"Teiserver.Connections.connect_user/1","doc":"","ref":"Teiserver.Connections.html#connect_user/1"},{"type":"function","title":"Teiserver.Connections.disconnect_user/1","doc":"","ref":"Teiserver.Connections.html#disconnect_user/1"},{"type":"function","title":"Teiserver.Connections.get_client/1","doc":"","ref":"Teiserver.Connections.html#get_client/1"},{"type":"function","title":"Teiserver.Connections.get_client_list/1","doc":"","ref":"Teiserver.Connections.html#get_client_list/1"},{"type":"function","title":"Teiserver.Connections.list_client_ids/0","doc":"","ref":"Teiserver.Connections.html#list_client_ids/0"},{"type":"function","title":"Teiserver.Connections.list_local_client_ids/0","doc":"","ref":"Teiserver.Connections.html#list_local_client_ids/0"},{"type":"function","title":"Teiserver.Connections.subscribe_to_client/1","doc":"","ref":"Teiserver.Connections.html#subscribe_to_client/1"},{"type":"function","title":"Teiserver.Connections.unsubscribe_from_client/1","doc":"","ref":"Teiserver.Connections.html#unsubscribe_from_client/1"},{"type":"function","title":"Teiserver.Connections.update_client/3","doc":"","ref":"Teiserver.Connections.html#update_client/3"},{"type":"function","title":"Teiserver.Connections.update_client_full/3","doc":"","ref":"Teiserver.Connections.html#update_client_full/3"},{"type":"function","title":"Teiserver.Connections.update_client_in_lobby/3","doc":"","ref":"Teiserver.Connections.html#update_client_in_lobby/3"},{"type":"module","title":"Teiserver.Game","doc":"The contextual module for:\n- `Teiserver.Game.Lobby`\n- `Teiserver.Game.Match`\n- `Teiserver.Game.MatchMembership`\n- `Teiserver.Game.MatchSettingType`\n- `Teiserver.Game.MatchSetting`\n- `Teiserver.Game.MatchResult`\n- `Teiserver.Game.ServerManagedLobby`","ref":"Teiserver.Game.html"},{"type":"function","title":"Teiserver.Game.add_client_to_lobby/2","doc":"","ref":"Teiserver.Game.html#add_client_to_lobby/2"},{"type":"function","title":"Teiserver.Game.can_add_client_to_lobby/2","doc":"","ref":"Teiserver.Game.html#can_add_client_to_lobby/2"},{"type":"function","title":"Teiserver.Game.can_add_client_to_lobby/3","doc":"","ref":"Teiserver.Game.html#can_add_client_to_lobby/3"},{"type":"function","title":"Teiserver.Game.change_match/2","doc":"","ref":"Teiserver.Game.html#change_match/2"},{"type":"function","title":"Teiserver.Game.change_match_membership/2","doc":"","ref":"Teiserver.Game.html#change_match_membership/2"},{"type":"function","title":"Teiserver.Game.change_match_setting/2","doc":"","ref":"Teiserver.Game.html#change_match_setting/2"},{"type":"function","title":"Teiserver.Game.change_match_setting_type/2","doc":"","ref":"Teiserver.Game.html#change_match_setting_type/2"},{"type":"function","title":"Teiserver.Game.change_match_type/2","doc":"","ref":"Teiserver.Game.html#change_match_type/2"},{"type":"function","title":"Teiserver.Game.close_lobby/1","doc":"","ref":"Teiserver.Game.html#close_lobby/1"},{"type":"function","title":"Teiserver.Game.create_many_match_memberships/1","doc":"","ref":"Teiserver.Game.html#create_many_match_memberships/1"},{"type":"function","title":"Teiserver.Game.create_many_match_settings/1","doc":"","ref":"Teiserver.Game.html#create_many_match_settings/1"},{"type":"function","title":"Teiserver.Game.create_match/1","doc":"","ref":"Teiserver.Game.html#create_match/1"},{"type":"function","title":"Teiserver.Game.create_match_membership/1","doc":"","ref":"Teiserver.Game.html#create_match_membership/1"},{"type":"function","title":"Teiserver.Game.create_match_setting/1","doc":"","ref":"Teiserver.Game.html#create_match_setting/1"},{"type":"function","title":"Teiserver.Game.create_match_setting_type/1","doc":"","ref":"Teiserver.Game.html#create_match_setting_type/1"},{"type":"function","title":"Teiserver.Game.create_match_type/1","doc":"","ref":"Teiserver.Game.html#create_match_type/1"},{"type":"function","title":"Teiserver.Game.cycle_lobby/1","doc":"","ref":"Teiserver.Game.html#cycle_lobby/1"},{"type":"function","title":"Teiserver.Game.delete_match/1","doc":"","ref":"Teiserver.Game.html#delete_match/1"},{"type":"function","title":"Teiserver.Game.delete_match_membership/1","doc":"","ref":"Teiserver.Game.html#delete_match_membership/1"},{"type":"function","title":"Teiserver.Game.delete_match_setting/1","doc":"","ref":"Teiserver.Game.html#delete_match_setting/1"},{"type":"function","title":"Teiserver.Game.delete_match_setting_type/1","doc":"","ref":"Teiserver.Game.html#delete_match_setting_type/1"},{"type":"function","title":"Teiserver.Game.delete_match_type/1","doc":"","ref":"Teiserver.Game.html#delete_match_type/1"},{"type":"function","title":"Teiserver.Game.end_match/2","doc":"","ref":"Teiserver.Game.html#end_match/2"},{"type":"function","title":"Teiserver.Game.get_lobby/1","doc":"","ref":"Teiserver.Game.html#get_lobby/1"},{"type":"function","title":"Teiserver.Game.get_lobby_attribute/2","doc":"","ref":"Teiserver.Game.html#get_lobby_attribute/2"},{"type":"function","title":"Teiserver.Game.get_lobby_summary/1","doc":"","ref":"Teiserver.Game.html#get_lobby_summary/1"},{"type":"function","title":"Teiserver.Game.get_match/2","doc":"","ref":"Teiserver.Game.html#get_match/2"},{"type":"function","title":"Teiserver.Game.get_match!/2","doc":"","ref":"Teiserver.Game.html#get_match!/2"},{"type":"function","title":"Teiserver.Game.get_match_membership/3","doc":"","ref":"Teiserver.Game.html#get_match_membership/3"},{"type":"function","title":"Teiserver.Game.get_match_membership!/3","doc":"","ref":"Teiserver.Game.html#get_match_membership!/3"},{"type":"function","title":"Teiserver.Game.get_match_setting/3","doc":"","ref":"Teiserver.Game.html#get_match_setting/3"},{"type":"function","title":"Teiserver.Game.get_match_setting!/3","doc":"","ref":"Teiserver.Game.html#get_match_setting!/3"},{"type":"function","title":"Teiserver.Game.get_match_setting_type/2","doc":"","ref":"Teiserver.Game.html#get_match_setting_type/2"},{"type":"function","title":"Teiserver.Game.get_match_setting_type!/2","doc":"","ref":"Teiserver.Game.html#get_match_setting_type!/2"},{"type":"function","title":"Teiserver.Game.get_match_settings_map/1","doc":"","ref":"Teiserver.Game.html#get_match_settings_map/1"},{"type":"function","title":"Teiserver.Game.get_match_type/2","doc":"","ref":"Teiserver.Game.html#get_match_type/2"},{"type":"function","title":"Teiserver.Game.get_match_type!/2","doc":"","ref":"Teiserver.Game.html#get_match_type!/2"},{"type":"function","title":"Teiserver.Game.get_or_create_match_setting_type/1","doc":"","ref":"Teiserver.Game.html#get_or_create_match_setting_type/1"},{"type":"function","title":"Teiserver.Game.get_or_create_match_type/1","doc":"","ref":"Teiserver.Game.html#get_or_create_match_type/1"},{"type":"function","title":"Teiserver.Game.list_lobby_ids/0","doc":"","ref":"Teiserver.Game.html#list_lobby_ids/0"},{"type":"function","title":"Teiserver.Game.list_local_lobby_ids/0","doc":"","ref":"Teiserver.Game.html#list_local_lobby_ids/0"},{"type":"function","title":"Teiserver.Game.list_match_memberships/1","doc":"","ref":"Teiserver.Game.html#list_match_memberships/1"},{"type":"function","title":"Teiserver.Game.list_match_setting_types/1","doc":"","ref":"Teiserver.Game.html#list_match_setting_types/1"},{"type":"function","title":"Teiserver.Game.list_match_settings/1","doc":"","ref":"Teiserver.Game.html#list_match_settings/1"},{"type":"function","title":"Teiserver.Game.list_match_types/1","doc":"","ref":"Teiserver.Game.html#list_match_types/1"},{"type":"function","title":"Teiserver.Game.list_matches/1","doc":"","ref":"Teiserver.Game.html#list_matches/1"},{"type":"function","title":"Teiserver.Game.lobby_name_acceptable?/1","doc":"","ref":"Teiserver.Game.html#lobby_name_acceptable?/1"},{"type":"function","title":"Teiserver.Game.lobby_start_match/1","doc":"","ref":"Teiserver.Game.html#lobby_start_match/1"},{"type":"function","title":"Teiserver.Game.open_lobby/2","doc":"","ref":"Teiserver.Game.html#open_lobby/2"},{"type":"function","title":"Teiserver.Game.remove_client_from_lobby/2","doc":"","ref":"Teiserver.Game.html#remove_client_from_lobby/2"},{"type":"function","title":"Teiserver.Game.start_match/1","doc":"","ref":"Teiserver.Game.html#start_match/1"},{"type":"function","title":"Teiserver.Game.stream_lobby_summaries/0","doc":"","ref":"Teiserver.Game.html#stream_lobby_summaries/0"},{"type":"function","title":"Teiserver.Game.stream_lobby_summaries/1","doc":"","ref":"Teiserver.Game.html#stream_lobby_summaries/1"},{"type":"function","title":"Teiserver.Game.subscribe_to_lobby/1","doc":"","ref":"Teiserver.Game.html#subscribe_to_lobby/1"},{"type":"function","title":"Teiserver.Game.unsubscribe_from_lobby/1","doc":"","ref":"Teiserver.Game.html#unsubscribe_from_lobby/1"},{"type":"function","title":"Teiserver.Game.update_lobby/2","doc":"","ref":"Teiserver.Game.html#update_lobby/2"},{"type":"function","title":"Teiserver.Game.update_match/2","doc":"","ref":"Teiserver.Game.html#update_match/2"},{"type":"function","title":"Teiserver.Game.update_match_membership/2","doc":"","ref":"Teiserver.Game.html#update_match_membership/2"},{"type":"function","title":"Teiserver.Game.update_match_setting/2","doc":"","ref":"Teiserver.Game.html#update_match_setting/2"},{"type":"function","title":"Teiserver.Game.update_match_setting_type/2","doc":"","ref":"Teiserver.Game.html#update_match_setting_type/2"},{"type":"function","title":"Teiserver.Game.update_match_type/2","doc":"","ref":"Teiserver.Game.html#update_match_type/2"},{"type":"module","title":"Teiserver.Logging","doc":"The contextual module for:\n- `Teiserver.Logging.AuditLog`\n- `Teiserver.Logging.CrashLog`\n- `Teiserver.Logging.UserLog`\n- `Teiserver.Logging.UsageLog`\n- `Teiserver.Logging.LobbyLog`\n- `Teiserver.Logging.MatchLog`\n- `Teiserver.Logging.EventLog`","ref":"Teiserver.Logging.html"},{"type":"module","title":"Teiserver.Matchmaking","doc":"The contextual module for:\n- `Teiserver.Matchmaking.Queue`\n- `Teiserver.Matchmaking.UserRating`\n- `Teiserver.Matchmaking.RatingLog`","ref":"Teiserver.Matchmaking.html"},{"type":"module","title":"Teiserver.Moderation","doc":"The contextual module for:\n- `Teiserver.Moderation.Report`\n- `Teiserver.Moderation.ReportGroup`\n- `Teiserver.Moderation.ReportGroupResponse`\n- `Teiserver.Moderation.Action`\n- `Teiserver.Moderation.Ban`\n- `Teiserver.Moderation.Appeal`\n- `Teiserver.Moderation.AppealResponse`","ref":"Teiserver.Moderation.html"},{"type":"module","title":"Teiserver.Settings","doc":"The contextual module for:\n- `Teiserver.Settings.ServerSetting`\n- `Teiserver.Settings.UserSetting`","ref":"Teiserver.Settings.html"},{"type":"function","title":"Teiserver.Settings.change_server_setting/2","doc":"","ref":"Teiserver.Settings.html#change_server_setting/2"},{"type":"function","title":"Teiserver.Settings.change_user_setting/2","doc":"","ref":"Teiserver.Settings.html#change_user_setting/2"},{"type":"function","title":"Teiserver.Settings.create_server_setting/1","doc":"","ref":"Teiserver.Settings.html#create_server_setting/1"},{"type":"function","title":"Teiserver.Settings.create_user_setting/1","doc":"","ref":"Teiserver.Settings.html#create_user_setting/1"},{"type":"function","title":"Teiserver.Settings.delete_server_setting/1","doc":"","ref":"Teiserver.Settings.html#delete_server_setting/1"},{"type":"function","title":"Teiserver.Settings.delete_user_setting/1","doc":"","ref":"Teiserver.Settings.html#delete_user_setting/1"},{"type":"function","title":"Teiserver.Settings.get_server_setting/2","doc":"","ref":"Teiserver.Settings.html#get_server_setting/2"},{"type":"function","title":"Teiserver.Settings.get_server_setting!/2","doc":"","ref":"Teiserver.Settings.html#get_server_setting!/2"},{"type":"function","title":"Teiserver.Settings.get_user_setting/2","doc":"","ref":"Teiserver.Settings.html#get_user_setting/2"},{"type":"function","title":"Teiserver.Settings.get_user_setting!/2","doc":"","ref":"Teiserver.Settings.html#get_user_setting!/2"},{"type":"function","title":"Teiserver.Settings.list_server_settings/0","doc":"","ref":"Teiserver.Settings.html#list_server_settings/0"},{"type":"function","title":"Teiserver.Settings.list_server_settings/1","doc":"","ref":"Teiserver.Settings.html#list_server_settings/1"},{"type":"function","title":"Teiserver.Settings.list_user_settings/0","doc":"","ref":"Teiserver.Settings.html#list_user_settings/0"},{"type":"function","title":"Teiserver.Settings.list_user_settings/1","doc":"","ref":"Teiserver.Settings.html#list_user_settings/1"},{"type":"function","title":"Teiserver.Settings.update_server_setting/2","doc":"","ref":"Teiserver.Settings.html#update_server_setting/2"},{"type":"function","title":"Teiserver.Settings.update_user_setting/2","doc":"","ref":"Teiserver.Settings.html#update_user_setting/2"},{"type":"module","title":"Teiserver.Telemetry","doc":"The contextual module for:\n- `Teiserver.Telemetry.SimpleGameEvent`\n- `Teiserver.Telemetry.SimpleServerEvent`\n- `Teiserver.Telemetry.SimpleLobbyEvent`\n- `Teiserver.Telemetry.SimpleUserEvent`\n- `Teiserver.Telemetry.SimpleClientEvent`\n- `Teiserver.Telemetry.SimpleAnonEvent`\n\n- `Teiserver.Telemetry.ComplexGameEvent`\n- `Teiserver.Telemetry.ComplexServerEvent`\n- `Teiserver.Telemetry.ComplexLobbyEvent`\n- `Teiserver.Telemetry.ComplexUserEvent`\n- `Teiserver.Telemetry.ComplexClientEvent`\n- `Teiserver.Telemetry.ComplexAnonEvent`","ref":"Teiserver.Telemetry.html"},{"type":"module","title":"Teiserver.Account.ExtraUserData","doc":"A blob of extra key-values for each user. Intended to be used as an easy to\nextend object which doesn't need to be queried quite as often as users.","ref":"Teiserver.Account.ExtraUserData.html"},{"type":"module","title":"Teiserver.Account.User","doc":"# User\nThe database representation of a person using/playing your game; not to be confused with `Teiserver.Connections.Client` which represents their logged in and online state.\n\n#","ref":"Teiserver.Account.User.html"},{"type":"module","title":"Attributes - Teiserver.Account.User","doc":"* `:name` - The name of the user\n* `:email` - The email of the user\n* `:password` - The encrypted password of the user; if you ever want to compare the password please make user of `valid_password?/2` (which can be called from `Teiserver.Account`).\n* `:groups` - A list of the groups possessed by the user, these are used to inform permissions\n* `:permissions` - A list of the things the user is allowed to do, typically derived from a combination of groups and restrictions\n* `:behaviour_score` - Numerical score representing the behaviour of the user, a high score means they are better behaved\n* `:trust_score` - Numerical score representing the trustworthiness and accuracy of their reporting of other users\n* `:social_score` - Numerical score representing how much the user is liked or disliked by other players in general\n* `:last_login_at` - DateTime of the last time they logged in when already logged out (so a user who is already logged in will not get this updated if they login with a 2nd application)\n* `:last_played_at` - DateTime of the last time they played\n* `:last_logout_at` - DateTime of the last time they logged out\n* `:restrictions` - A list of the restrictions applied to the user account\n* `:restricted_until` - DateTime of when the restrictions to the user may need to be revised (e.g. when a moderation action expires)\n* `:shadow_banned?` - Boolean flag for spambots and trolls, when set to true the user should be allowed to see things as if fully logged in and be sent \"success\" type messages. In reality they are not able to effect change.","ref":"Teiserver.Account.User.html#module-attributes"},{"type":"function","title":"Teiserver.Account.User.calculate_user_permissions/1","doc":"Given a User changeset, calculate what the permissions should be and update the changeset","ref":"Teiserver.Account.User.html#calculate_user_permissions/1"},{"type":"function","title":"Teiserver.Account.User.default_calculate_user_permissions/1","doc":"The default method used to define match types\n\nCan be over-ridden using the config [fn_calculate_user_permissions](config.html#fn_calculate_user_permissions)","ref":"Teiserver.Account.User.html#default_calculate_user_permissions/1"},{"type":"function","title":"Teiserver.Account.User.valid_password?/2","doc":"Provides a secure way to verify a password to prevent timing attacks.","ref":"Teiserver.Account.User.html#valid_password?/2"},{"type":"function","title":"Examples - Teiserver.Account.User.valid_password?/2","doc":"iex> valid_password?(plaintext, user.password)\n    true\n\n    iex> valid_password?(\"bad_password\", user.password)\n    false","ref":"Teiserver.Account.User.html#valid_password?/2-examples"},{"type":"type","title":"Teiserver.Account.User.id/0","doc":"","ref":"Teiserver.Account.User.html#t:id/0"},{"type":"type","title":"Teiserver.Account.User.t/0","doc":"","ref":"Teiserver.Account.User.html#t:t/0"},{"type":"module","title":"Teiserver.Account.UserLib","doc":"Library of user related functions.","ref":"Teiserver.Account.UserLib.html"},{"type":"function","title":"Teiserver.Account.UserLib.allow?/2","doc":"Tests if a User or user_id has all of the required permissions.\n\nIf the user doesn't exist you will get back a failure.","ref":"Teiserver.Account.UserLib.html#allow?/2"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.allow?/2","doc":"iex> allow?(123, \"Permission\")\n  true\n\n  iex> allow?(123, \"NotPermission\")\n  false","ref":"Teiserver.Account.UserLib.html#allow?/2-examples"},{"type":"function","title":"Teiserver.Account.UserLib.change_user/2","doc":"Returns an `%Ecto.Changeset{}` for tracking user changes.","ref":"Teiserver.Account.UserLib.html#change_user/2"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.change_user/2","doc":"iex> change_user(user)\n    %Ecto.Changeset{data: %User{}}","ref":"Teiserver.Account.UserLib.html#change_user/2-examples"},{"type":"function","title":"Teiserver.Account.UserLib.create_user/1","doc":"Creates a user with no checks, use this for system users or automated processes; for user registration make use of `register_user/1`.","ref":"Teiserver.Account.UserLib.html#create_user/1"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.create_user/1","doc":"iex> create_user(%{field: value})\n    {:ok, %User{}}\n\n    iex> create_user(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Account.UserLib.html#create_user/1-examples"},{"type":"function","title":"Teiserver.Account.UserLib.default_user_name_acceptable?/1","doc":"","ref":"Teiserver.Account.UserLib.html#default_user_name_acceptable?/1"},{"type":"function","title":"Teiserver.Account.UserLib.delete_user/1","doc":"Deletes a user.","ref":"Teiserver.Account.UserLib.html#delete_user/1"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.delete_user/1","doc":"iex> delete_user(user)\n    {:ok, %User{}}\n\n    iex> delete_user(user)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Account.UserLib.html#delete_user/1-examples"},{"type":"function","title":"Teiserver.Account.UserLib.generate_password/0","doc":"Generates a strong, though not very human readable, password.","ref":"Teiserver.Account.UserLib.html#generate_password/0"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.generate_password/0","doc":"iex> generate_password()\n  \"d52r8i5BhA6xBtmp7ElHI3Y/U/qztw2jUkgdeoZijWBEYzTf5DSBR5N87283WDiA\"","ref":"Teiserver.Account.UserLib.html#generate_password/0-examples"},{"type":"function","title":"Teiserver.Account.UserLib.get_user/2","doc":"Gets a single user. Can take additional arguments for the query.\n\nReturns nil if the User does not exist.","ref":"Teiserver.Account.UserLib.html#get_user/2"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.get_user/2","doc":"iex> get_user(123)\n    %User{}\n\n    iex> get_user(456)\n    nil\n\n    iex> get_user(123, preload: [:extra_user_data])\n    %User{}","ref":"Teiserver.Account.UserLib.html#get_user/2-examples"},{"type":"function","title":"Teiserver.Account.UserLib.get_user!/2","doc":"Gets a single user.\n\nRaises `Ecto.NoResultsError` if the User does not exist.","ref":"Teiserver.Account.UserLib.html#get_user!/2"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.get_user!/2","doc":"iex> get_user!(123)\n    %User{}\n\n    iex> get_user!(456)\n    ** (Ecto.NoResultsError)","ref":"Teiserver.Account.UserLib.html#get_user!/2-examples"},{"type":"function","title":"Teiserver.Account.UserLib.get_user_by_email/1","doc":"Gets a single user by their email. If no user is found, returns `nil`.","ref":"Teiserver.Account.UserLib.html#get_user_by_email/1"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.get_user_by_email/1","doc":"iex> get_user_by_email(\"noodle@teiserver\")\n    %User{}\n\n    iex> get_user_by_email(\"nobody@nowhere\")\n    nil","ref":"Teiserver.Account.UserLib.html#get_user_by_email/1-examples"},{"type":"function","title":"Teiserver.Account.UserLib.get_user_by_id/1","doc":"Gets a single user by their user_id. If no user is found, returns `nil`.","ref":"Teiserver.Account.UserLib.html#get_user_by_id/1"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.get_user_by_id/1","doc":"iex> get_user_by_id(123)\n    %User{}\n\n    iex> get_user_by_id(456)\n    nil","ref":"Teiserver.Account.UserLib.html#get_user_by_id/1-examples"},{"type":"function","title":"Teiserver.Account.UserLib.get_user_by_name/1","doc":"Gets a single user by their name. If no user is found, returns `nil`.","ref":"Teiserver.Account.UserLib.html#get_user_by_name/1"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.get_user_by_name/1","doc":"iex> get_user_by_name(\"noodle\")\n    %User{}\n\n    iex> get_user_by_name(\"nobody\")\n    nil","ref":"Teiserver.Account.UserLib.html#get_user_by_name/1-examples"},{"type":"function","title":"Teiserver.Account.UserLib.list_users/1","doc":"Returns the list of users.","ref":"Teiserver.Account.UserLib.html#list_users/1"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.list_users/1","doc":"iex> list_users()\n    [%User{}, ...]","ref":"Teiserver.Account.UserLib.html#list_users/1-examples"},{"type":"function","title":"Teiserver.Account.UserLib.register_user/1","doc":"Creates a user specifically via the registration changeset, you should use this for user-registration and `create_user/1` for system accounts or automated processes.","ref":"Teiserver.Account.UserLib.html#register_user/1"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.register_user/1","doc":"iex> register_user(%{field: value})\n    {:ok, %User{}}\n\n    iex> register_user(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Account.UserLib.html#register_user/1-examples"},{"type":"function","title":"Teiserver.Account.UserLib.restricted?/2","doc":"Tests if a User or user_id has any of the listed restrictions applied to their account.\n- Returns `true` to indicate the user is restricted\n- Returns `false` to indicate the user is not restricted\n\nIf the user doesn't exist you will get back a true.","ref":"Teiserver.Account.UserLib.html#restricted?/2"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.restricted?/2","doc":"iex> restricted?(123, \"Banned\")\n  true\n\n  iex> restricted?(123, \"NotRestriction\")\n  false","ref":"Teiserver.Account.UserLib.html#restricted?/2-examples"},{"type":"function","title":"Teiserver.Account.UserLib.update_limited_user/2","doc":"Updates a user but only the fields users are allowed to alter themselves.","ref":"Teiserver.Account.UserLib.html#update_limited_user/2"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.update_limited_user/2","doc":"iex> update_limited_user(user, %{field: new_value})\n    {:ok, %User{}}\n\n    iex> update_limited_user(user, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Account.UserLib.html#update_limited_user/2-examples"},{"type":"function","title":"Teiserver.Account.UserLib.update_password/2","doc":"Updates a user's password.","ref":"Teiserver.Account.UserLib.html#update_password/2"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.update_password/2","doc":"iex> update_password(user, %{field: new_value})\n    {:ok, %User{}}\n\n    iex> update_password(user, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Account.UserLib.html#update_password/2-examples"},{"type":"function","title":"Teiserver.Account.UserLib.update_user/2","doc":"Updates a user.","ref":"Teiserver.Account.UserLib.html#update_user/2"},{"type":"function","title":"Examples - Teiserver.Account.UserLib.update_user/2","doc":"iex> update_user(user, %{field: new_value})\n    {:ok, %User{}}\n\n    iex> update_user(user, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Account.UserLib.html#update_user/2-examples"},{"type":"function","title":"Teiserver.Account.UserLib.user_name_acceptable?/1","doc":"Tests is the user name is acceptable. Can be over-ridden using the config [fn_user_name_acceptor](config.html#fn_user_name_acceptor)","ref":"Teiserver.Account.UserLib.html#user_name_acceptable?/1"},{"type":"function","title":"Teiserver.Account.UserLib.valid_password?/2","doc":"Takes a user, a plaintext password and returns a boolean if the password is\ncorrect for the user. Note it does this via a secure method to prevent timing\nattacks, never manually verify the password with standard string comparison.","ref":"Teiserver.Account.UserLib.html#valid_password?/2"},{"type":"module","title":"Teiserver.Communication.DirectMessage","doc":"# DirectMessage\nA (text) message from one user to another.\n\n#","ref":"Teiserver.Communication.DirectMessage.html"},{"type":"module","title":"Attributes - Teiserver.Communication.DirectMessage","doc":"* `:content` - The text content of the message\n* `:inserted_at` - The datetime the message was sent\n* `:delivered?` - Tracks if the message has been delivered to the user or not\n* `:read?` - Tracks if the message has been read by the user or not\n* `:sender_id` - The `Teiserver.Account.User` who sent the message\n* `:to_id` - The `Teiserver.Account.User` the message was sent to","ref":"Teiserver.Communication.DirectMessage.html#module-attributes"},{"type":"type","title":"Teiserver.Communication.DirectMessage.id/0","doc":"","ref":"Teiserver.Communication.DirectMessage.html#t:id/0"},{"type":"type","title":"Teiserver.Communication.DirectMessage.t/0","doc":"","ref":"Teiserver.Communication.DirectMessage.html#t:t/0"},{"type":"module","title":"Teiserver.Communication.DirectMessageLib","doc":"Library of direct_message related functions.","ref":"Teiserver.Communication.DirectMessageLib.html"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.change_direct_message/2","doc":"Returns an `%Ecto.Changeset{}` for tracking direct_message changes.","ref":"Teiserver.Communication.DirectMessageLib.html#change_direct_message/2"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.change_direct_message/2","doc":"iex> change_direct_message(direct_message)\n    %Ecto.Changeset{data: %DirectMessage{}}","ref":"Teiserver.Communication.DirectMessageLib.html#change_direct_message/2-examples"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.create_direct_message/1","doc":"Creates a direct_message.","ref":"Teiserver.Communication.DirectMessageLib.html#create_direct_message/1"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.create_direct_message/1","doc":"iex> create_direct_message(%{field: value})\n    {:ok, %DirectMessage{}}\n\n    iex> create_direct_message(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.DirectMessageLib.html#create_direct_message/1-examples"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.delete_direct_message/1","doc":"Deletes a direct_message.","ref":"Teiserver.Communication.DirectMessageLib.html#delete_direct_message/1"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.delete_direct_message/1","doc":"iex> delete_direct_message(direct_message)\n    {:ok, %DirectMessage{}}\n\n    iex> delete_direct_message(direct_message)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.DirectMessageLib.html#delete_direct_message/1-examples"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.get_direct_message/2","doc":"Gets a single direct_message.\n\nReturns nil if the DirectMessage does not exist.","ref":"Teiserver.Communication.DirectMessageLib.html#get_direct_message/2"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.get_direct_message/2","doc":"iex> get_direct_message(123)\n    %DirectMessage{}\n\n    iex> get_direct_message(456)\n    nil","ref":"Teiserver.Communication.DirectMessageLib.html#get_direct_message/2-examples"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.get_direct_message!/2","doc":"Gets a single direct_message.\n\nRaises `Ecto.NoResultsError` if the DirectMessage does not exist.","ref":"Teiserver.Communication.DirectMessageLib.html#get_direct_message!/2"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.get_direct_message!/2","doc":"iex> get_direct_message!(123)\n    %DirectMessage{}\n\n    iex> get_direct_message!(456)\n    ** (Ecto.NoResultsError)","ref":"Teiserver.Communication.DirectMessageLib.html#get_direct_message!/2-examples"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.list_direct_messages/1","doc":"Returns the list of direct_messages.","ref":"Teiserver.Communication.DirectMessageLib.html#list_direct_messages/1"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.list_direct_messages/1","doc":"iex> list_direct_messages()\n    [%DirectMessage{}, ...]","ref":"Teiserver.Communication.DirectMessageLib.html#list_direct_messages/1-examples"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.list_direct_messages_between_users/3","doc":"Returns the list of direct_messages sent between specific users.","ref":"Teiserver.Communication.DirectMessageLib.html#list_direct_messages_between_users/3"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.list_direct_messages_between_users/3","doc":"iex> list_direct_messages_between_users(user1_id, user2_id)\n    [%DirectMessage{}, ...]\n\n    iex> list_direct_messages_between_users(user1_id, user2_id, limit: 10, order_by: [\"Newest first\"])\n    [%DirectMessage{}, ...]","ref":"Teiserver.Communication.DirectMessageLib.html#list_direct_messages_between_users/3-examples"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.list_direct_messages_for_user/2","doc":"Returns the list of direct_messages for a specific user (to and from).","ref":"Teiserver.Communication.DirectMessageLib.html#list_direct_messages_for_user/2"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.list_direct_messages_for_user/2","doc":"iex> list_direct_messages_for_user(user_id)\n    [%DirectMessage{}, ...]","ref":"Teiserver.Communication.DirectMessageLib.html#list_direct_messages_for_user/2-examples"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.list_direct_messages_from_user/2","doc":"Returns the list of direct_messages sent from a specific user.","ref":"Teiserver.Communication.DirectMessageLib.html#list_direct_messages_from_user/2"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.list_direct_messages_from_user/2","doc":"iex> list_direct_messages_from_user(user_id)\n    [%DirectMessage{}, ...]","ref":"Teiserver.Communication.DirectMessageLib.html#list_direct_messages_from_user/2-examples"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.list_direct_messages_from_user_to_user/3","doc":"Returns the list of direct_messages sent from a specific user to a specific user.","ref":"Teiserver.Communication.DirectMessageLib.html#list_direct_messages_from_user_to_user/3"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.list_direct_messages_from_user_to_user/3","doc":"iex> list_direct_messages_from_user_to_user(sender_id, to_id)\n    [%DirectMessage{}, ...]","ref":"Teiserver.Communication.DirectMessageLib.html#list_direct_messages_from_user_to_user/3-examples"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.list_direct_messages_to_user/2","doc":"Returns the list of direct_messages sent to a specific user.","ref":"Teiserver.Communication.DirectMessageLib.html#list_direct_messages_to_user/2"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.list_direct_messages_to_user/2","doc":"iex> list_direct_messages_to_user(user_id)\n    [%DirectMessage{}, ...]","ref":"Teiserver.Communication.DirectMessageLib.html#list_direct_messages_to_user/2-examples"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.send_direct_message/4","doc":"- Creates a direct message\n- If successful generate a pubsub message","ref":"Teiserver.Communication.DirectMessageLib.html#send_direct_message/4"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.send_direct_message/4","doc":"iex> send_direct_message(123, 124, \"Message content\")\n    {:ok, %DirectMessage{}}\n\n    iex> send_direct_message(456, 457, \"Message content\")\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.DirectMessageLib.html#send_direct_message/4-examples"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.subscribe_to_user_messaging/1","doc":"Joins your process to match messages","ref":"Teiserver.Communication.DirectMessageLib.html#subscribe_to_user_messaging/1"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.unsubscribe_from_user_messaging/1","doc":"Removes your process from a match's messages","ref":"Teiserver.Communication.DirectMessageLib.html#unsubscribe_from_user_messaging/1"},{"type":"function","title":"Teiserver.Communication.DirectMessageLib.update_direct_message/2","doc":"Updates a direct_message.","ref":"Teiserver.Communication.DirectMessageLib.html#update_direct_message/2"},{"type":"function","title":"Examples - Teiserver.Communication.DirectMessageLib.update_direct_message/2","doc":"iex> update_direct_message(direct_message, %{field: new_value})\n    {:ok, %DirectMessage{}}\n\n    iex> update_direct_message(direct_message, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.DirectMessageLib.html#update_direct_message/2-examples"},{"type":"module","title":"Teiserver.Communication.MatchMessage","doc":"# MatchMessage\nA message sent either in a lobby or in a game and mirrored in the lobby\n\n#","ref":"Teiserver.Communication.MatchMessage.html"},{"type":"module","title":"Attributes - Teiserver.Communication.MatchMessage","doc":"* `:content` - The (textual) contents of the message\n* `:inserted_at` - The time the message was inserted\n* `:sender_id` - The `Teiserver.Account.User` who sent the message\n* `:match_id` - The id of the `Teiserver.Game.Match` the message was sent in","ref":"Teiserver.Communication.MatchMessage.html#module-attributes"},{"type":"type","title":"Teiserver.Communication.MatchMessage.id/0","doc":"","ref":"Teiserver.Communication.MatchMessage.html#t:id/0"},{"type":"type","title":"Teiserver.Communication.MatchMessage.t/0","doc":"","ref":"Teiserver.Communication.MatchMessage.html#t:t/0"},{"type":"module","title":"Teiserver.Communication.MatchMessageLib","doc":"Library of match_message related functions.","ref":"Teiserver.Communication.MatchMessageLib.html"},{"type":"function","title":"Teiserver.Communication.MatchMessageLib.change_match_message/2","doc":"Returns an `%Ecto.Changeset{}` for tracking match_message changes.","ref":"Teiserver.Communication.MatchMessageLib.html#change_match_message/2"},{"type":"function","title":"Examples - Teiserver.Communication.MatchMessageLib.change_match_message/2","doc":"iex> change_match_message(match_message)\n    %Ecto.Changeset{data: %MatchMessage{}}","ref":"Teiserver.Communication.MatchMessageLib.html#change_match_message/2-examples"},{"type":"function","title":"Teiserver.Communication.MatchMessageLib.create_match_message/1","doc":"Creates a match_message.","ref":"Teiserver.Communication.MatchMessageLib.html#create_match_message/1"},{"type":"function","title":"Examples - Teiserver.Communication.MatchMessageLib.create_match_message/1","doc":"iex> create_match_message(%{field: value})\n    {:ok, %MatchMessage{}}\n\n    iex> create_match_message(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.MatchMessageLib.html#create_match_message/1-examples"},{"type":"function","title":"Teiserver.Communication.MatchMessageLib.delete_match_message/1","doc":"Deletes a match_message.","ref":"Teiserver.Communication.MatchMessageLib.html#delete_match_message/1"},{"type":"function","title":"Examples - Teiserver.Communication.MatchMessageLib.delete_match_message/1","doc":"iex> delete_match_message(match_message)\n    {:ok, %MatchMessage{}}\n\n    iex> delete_match_message(match_message)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.MatchMessageLib.html#delete_match_message/1-examples"},{"type":"function","title":"Teiserver.Communication.MatchMessageLib.get_match_message/2","doc":"Gets a single match_message.\n\nReturns nil if the MatchMessage does not exist.","ref":"Teiserver.Communication.MatchMessageLib.html#get_match_message/2"},{"type":"function","title":"Examples - Teiserver.Communication.MatchMessageLib.get_match_message/2","doc":"iex> get_match_message(\"uuid\")\n    %MatchMessage{}\n\n    iex> get_match_message(\"uuid\")\n    nil","ref":"Teiserver.Communication.MatchMessageLib.html#get_match_message/2-examples"},{"type":"function","title":"Teiserver.Communication.MatchMessageLib.get_match_message!/2","doc":"Gets a single match_message.\n\nRaises `Ecto.NoResultsError` if the MatchMessage does not exist.","ref":"Teiserver.Communication.MatchMessageLib.html#get_match_message!/2"},{"type":"function","title":"Examples - Teiserver.Communication.MatchMessageLib.get_match_message!/2","doc":"iex> get_match_message!(\"uuid\")\n    %MatchMessage{}\n\n    iex> get_match_message!(\"uuid\")\n    ** (Ecto.NoResultsError)","ref":"Teiserver.Communication.MatchMessageLib.html#get_match_message!/2-examples"},{"type":"function","title":"Teiserver.Communication.MatchMessageLib.list_match_messages/1","doc":"Returns the list of match_messages.","ref":"Teiserver.Communication.MatchMessageLib.html#list_match_messages/1"},{"type":"function","title":"Examples - Teiserver.Communication.MatchMessageLib.list_match_messages/1","doc":"iex> list_match_messages()\n    [%MatchMessage{}, ...]","ref":"Teiserver.Communication.MatchMessageLib.html#list_match_messages/1-examples"},{"type":"function","title":"Teiserver.Communication.MatchMessageLib.list_recent_match_messages/2","doc":"Returns a list of messages from a match ordered as the newest first.\n\nIn the event of there being ne messages for a match of that ID the function will return an empty list.","ref":"Teiserver.Communication.MatchMessageLib.html#list_recent_match_messages/2"},{"type":"function","title":"Examples - Teiserver.Communication.MatchMessageLib.list_recent_match_messages/2","doc":"iex> list_recent_match_messages(\"uuid\")\n    [%MatchMessage{}, ...]\n\n    iex> list_recent_match_messages(\"uuid\")\n    []","ref":"Teiserver.Communication.MatchMessageLib.html#list_recent_match_messages/2-examples"},{"type":"function","title":"Teiserver.Communication.MatchMessageLib.send_lobby_message/3","doc":"Wraps `send_match_message/3` to send a message when in the lobby instead of the match\n\n  - Creates a match message\n  - If successful generate a pubsub message for both the Match and the Lobby","ref":"Teiserver.Communication.MatchMessageLib.html#send_lobby_message/3"},{"type":"function","title":"Examples - Teiserver.Communication.MatchMessageLib.send_lobby_message/3","doc":"iex> send_lobby_message(\"uuid\", \"uuid\", \"Message content\")\n      {:ok, %MatchMessage{}}\n\n      iex> send_lobby_message(\"uuid\", \"uuid\", \"Message content\")\n      {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.MatchMessageLib.html#send_lobby_message/3-examples"},{"type":"function","title":"Teiserver.Communication.MatchMessageLib.send_match_message/4","doc":"- Creates a match message\n  - If successful generate a pubsub message","ref":"Teiserver.Communication.MatchMessageLib.html#send_match_message/4"},{"type":"function","title":"Examples - Teiserver.Communication.MatchMessageLib.send_match_message/4","doc":"iex> send_match_message(\"uuid\", \"uuid\", \"Message content\")\n      {:ok, %MatchMessage{}}\n\n      iex> send_match_message(\"uuid\", \"uuid\", \"Message content\")\n      {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.MatchMessageLib.html#send_match_message/4-examples"},{"type":"function","title":"Teiserver.Communication.MatchMessageLib.subscribe_to_match_messages/1","doc":"Joins your process to match messages","ref":"Teiserver.Communication.MatchMessageLib.html#subscribe_to_match_messages/1"},{"type":"function","title":"Teiserver.Communication.MatchMessageLib.unsubscribe_from_match_messages/1","doc":"Removes your process from a match's messages","ref":"Teiserver.Communication.MatchMessageLib.html#unsubscribe_from_match_messages/1"},{"type":"function","title":"Teiserver.Communication.MatchMessageLib.update_match_message/2","doc":"Updates a match_message.","ref":"Teiserver.Communication.MatchMessageLib.html#update_match_message/2"},{"type":"function","title":"Examples - Teiserver.Communication.MatchMessageLib.update_match_message/2","doc":"iex> update_match_message(match_message, %{field: new_value})\n    {:ok, %MatchMessage{}}\n\n    iex> update_match_message(match_message, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.MatchMessageLib.html#update_match_message/2-examples"},{"type":"module","title":"Teiserver.Communication.Room","doc":"# Room\nAn object representing a chat room.\n\n#","ref":"Teiserver.Communication.Room.html"},{"type":"module","title":"Attributes - Teiserver.Communication.Room","doc":"* `:name` - The room name","ref":"Teiserver.Communication.Room.html#module-attributes"},{"type":"type","title":"Teiserver.Communication.Room.id/0","doc":"","ref":"Teiserver.Communication.Room.html#t:id/0"},{"type":"type","title":"Teiserver.Communication.Room.name/0","doc":"","ref":"Teiserver.Communication.Room.html#t:name/0"},{"type":"type","title":"Teiserver.Communication.Room.name_or_id/0","doc":"","ref":"Teiserver.Communication.Room.html#t:name_or_id/0"},{"type":"type","title":"Teiserver.Communication.Room.t/0","doc":"","ref":"Teiserver.Communication.Room.html#t:t/0"},{"type":"module","title":"Teiserver.Communication.RoomLib","doc":"Library of room related functions.","ref":"Teiserver.Communication.RoomLib.html"},{"type":"function","title":"Teiserver.Communication.RoomLib.change_room/2","doc":"Returns an `%Ecto.Changeset{}` for tracking room changes.","ref":"Teiserver.Communication.RoomLib.html#change_room/2"},{"type":"function","title":"Examples - Teiserver.Communication.RoomLib.change_room/2","doc":"iex> change_room(room)\n    %Ecto.Changeset{data: %Room{}}","ref":"Teiserver.Communication.RoomLib.html#change_room/2-examples"},{"type":"function","title":"Teiserver.Communication.RoomLib.create_room/1","doc":"Creates a room.","ref":"Teiserver.Communication.RoomLib.html#create_room/1"},{"type":"function","title":"Examples - Teiserver.Communication.RoomLib.create_room/1","doc":"iex> create_room(%{field: value})\n    {:ok, %Room{}}\n\n    iex> create_room(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.RoomLib.html#create_room/1-examples"},{"type":"function","title":"Teiserver.Communication.RoomLib.delete_room/1","doc":"Deletes a room.","ref":"Teiserver.Communication.RoomLib.html#delete_room/1"},{"type":"function","title":"Examples - Teiserver.Communication.RoomLib.delete_room/1","doc":"iex> delete_room(room)\n    {:ok, %Room{}}\n\n    iex> delete_room(room)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.RoomLib.html#delete_room/1-examples"},{"type":"function","title":"Teiserver.Communication.RoomLib.get_or_create_room/1","doc":"Gets a room of a given name, if the room does not exist it is created.","ref":"Teiserver.Communication.RoomLib.html#get_or_create_room/1"},{"type":"function","title":"Examples - Teiserver.Communication.RoomLib.get_or_create_room/1","doc":"iex> get_or_create_room(\"name\")\n    %Room{}\n\n    iex> get_or_create_room(\"not a room\"\")\n    %Room{}","ref":"Teiserver.Communication.RoomLib.html#get_or_create_room/1-examples"},{"type":"function","title":"Teiserver.Communication.RoomLib.get_room/2","doc":"Gets a single room.\n\nReturns nil if the Room does not exist.","ref":"Teiserver.Communication.RoomLib.html#get_room/2"},{"type":"function","title":"Examples - Teiserver.Communication.RoomLib.get_room/2","doc":"iex> get_room(123)\n    %Room{}\n\n    iex> get_room(456)\n    nil","ref":"Teiserver.Communication.RoomLib.html#get_room/2-examples"},{"type":"function","title":"Teiserver.Communication.RoomLib.get_room!/2","doc":"Gets a single room.\n\nRaises `Ecto.NoResultsError` if the Room does not exist.","ref":"Teiserver.Communication.RoomLib.html#get_room!/2"},{"type":"function","title":"Examples - Teiserver.Communication.RoomLib.get_room!/2","doc":"iex> get_room!(123)\n    %Room{}\n\n    iex> get_room!(456)\n    ** (Ecto.NoResultsError)","ref":"Teiserver.Communication.RoomLib.html#get_room!/2-examples"},{"type":"function","title":"Teiserver.Communication.RoomLib.get_room_by_name_or_id/1","doc":"Gets a single room by name or id (must be an integer).\n\nReturns nil if the Room does not exist.","ref":"Teiserver.Communication.RoomLib.html#get_room_by_name_or_id/1"},{"type":"function","title":"Examples - Teiserver.Communication.RoomLib.get_room_by_name_or_id/1","doc":"iex> get_room_by_name_or_id(\"main\")\n    %Room{}\n\n    iex> get_room_by_name_or_id(123)\n    %Room{}\n\n    iex> get_room_by_name_or_id(\"not a name\")\n    nil\n\n    iex> get_room_by_name_or_id(456)\n    nil","ref":"Teiserver.Communication.RoomLib.html#get_room_by_name_or_id/1-examples"},{"type":"function","title":"Teiserver.Communication.RoomLib.list_rooms/1","doc":"Returns the list of rooms.","ref":"Teiserver.Communication.RoomLib.html#list_rooms/1"},{"type":"function","title":"Examples - Teiserver.Communication.RoomLib.list_rooms/1","doc":"iex> list_rooms()\n    [%Room{}, ...]","ref":"Teiserver.Communication.RoomLib.html#list_rooms/1-examples"},{"type":"function","title":"Teiserver.Communication.RoomLib.update_room/2","doc":"Updates a room.","ref":"Teiserver.Communication.RoomLib.html#update_room/2"},{"type":"function","title":"Examples - Teiserver.Communication.RoomLib.update_room/2","doc":"iex> update_room(room, %{field: new_value})\n    {:ok, %Room{}}\n\n    iex> update_room(room, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.RoomLib.html#update_room/2-examples"},{"type":"module","title":"Teiserver.Communication.RoomMessage","doc":"# RoomMessage\nA message sent sender a user into a chat room.\n\n#","ref":"Teiserver.Communication.RoomMessage.html"},{"type":"module","title":"Attributes - Teiserver.Communication.RoomMessage","doc":"* `:content` - The (textual) contents of the message\n* `:inserted_at` - The time the message was inserted\n* `:sender_id` - The `Teiserver.Account.User` who sent the message\n* `:room_id` - The id of the `Teiserver.Communication.Room` the message was sent in","ref":"Teiserver.Communication.RoomMessage.html#module-attributes"},{"type":"type","title":"Teiserver.Communication.RoomMessage.id/0","doc":"","ref":"Teiserver.Communication.RoomMessage.html#t:id/0"},{"type":"type","title":"Teiserver.Communication.RoomMessage.t/0","doc":"","ref":"Teiserver.Communication.RoomMessage.html#t:t/0"},{"type":"module","title":"Teiserver.Communication.RoomMessageLib","doc":"Library of room_message related functions.","ref":"Teiserver.Communication.RoomMessageLib.html"},{"type":"function","title":"Teiserver.Communication.RoomMessageLib.change_room_message/2","doc":"Returns an `%Ecto.Changeset{}` for tracking room_message changes.","ref":"Teiserver.Communication.RoomMessageLib.html#change_room_message/2"},{"type":"function","title":"Examples - Teiserver.Communication.RoomMessageLib.change_room_message/2","doc":"iex> change_room_message(room_message)\n    %Ecto.Changeset{data: %RoomMessage{}}","ref":"Teiserver.Communication.RoomMessageLib.html#change_room_message/2-examples"},{"type":"function","title":"Teiserver.Communication.RoomMessageLib.create_room_message/1","doc":"Creates a room_message.","ref":"Teiserver.Communication.RoomMessageLib.html#create_room_message/1"},{"type":"function","title":"Examples - Teiserver.Communication.RoomMessageLib.create_room_message/1","doc":"iex> create_room_message(%{field: value})\n    {:ok, %RoomMessage{}}\n\n    iex> create_room_message(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.RoomMessageLib.html#create_room_message/1-examples"},{"type":"function","title":"Teiserver.Communication.RoomMessageLib.delete_room_message/1","doc":"Deletes a room_message.","ref":"Teiserver.Communication.RoomMessageLib.html#delete_room_message/1"},{"type":"function","title":"Examples - Teiserver.Communication.RoomMessageLib.delete_room_message/1","doc":"iex> delete_room_message(room_message)\n    {:ok, %RoomMessage{}}\n\n    iex> delete_room_message(room_message)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.RoomMessageLib.html#delete_room_message/1-examples"},{"type":"function","title":"Teiserver.Communication.RoomMessageLib.get_room_message/2","doc":"Gets a single room_message.\n\nReturns nil if the RoomMessage does not exist.","ref":"Teiserver.Communication.RoomMessageLib.html#get_room_message/2"},{"type":"function","title":"Examples - Teiserver.Communication.RoomMessageLib.get_room_message/2","doc":"iex> get_room_message(123)\n    %RoomMessage{}\n\n    iex> get_room_message(456)\n    nil","ref":"Teiserver.Communication.RoomMessageLib.html#get_room_message/2-examples"},{"type":"function","title":"Teiserver.Communication.RoomMessageLib.get_room_message!/2","doc":"Gets a single room_message.\n\nRaises `Ecto.NoResultsError` if the RoomMessage does not exist.","ref":"Teiserver.Communication.RoomMessageLib.html#get_room_message!/2"},{"type":"function","title":"Examples - Teiserver.Communication.RoomMessageLib.get_room_message!/2","doc":"iex> get_room_message!(123)\n    %RoomMessage{}\n\n    iex> get_room_message!(456)\n    ** (Ecto.NoResultsError)","ref":"Teiserver.Communication.RoomMessageLib.html#get_room_message!/2-examples"},{"type":"function","title":"Teiserver.Communication.RoomMessageLib.list_recent_room_messages/2","doc":"Returns a list of messages from a room ordered as the newest first.\n\nIn the event of there being ne messages for a room of that ID the function will return an empty list.","ref":"Teiserver.Communication.RoomMessageLib.html#list_recent_room_messages/2"},{"type":"function","title":"Examples - Teiserver.Communication.RoomMessageLib.list_recent_room_messages/2","doc":"iex> list_recent_room_messages(123)\n    [%RoomMessage{}, ...]\n\n    iex> list_recent_room_messages(456)\n    []","ref":"Teiserver.Communication.RoomMessageLib.html#list_recent_room_messages/2-examples"},{"type":"function","title":"Teiserver.Communication.RoomMessageLib.list_room_messages/1","doc":"Returns the list of room_messages.","ref":"Teiserver.Communication.RoomMessageLib.html#list_room_messages/1"},{"type":"function","title":"Examples - Teiserver.Communication.RoomMessageLib.list_room_messages/1","doc":"iex> list_room_messages()\n    [%RoomMessage{}, ...]","ref":"Teiserver.Communication.RoomMessageLib.html#list_room_messages/1-examples"},{"type":"function","title":"Teiserver.Communication.RoomMessageLib.send_room_message/4","doc":"- Creates a room message\n  - If successful generate a pubsub message","ref":"Teiserver.Communication.RoomMessageLib.html#send_room_message/4"},{"type":"function","title":"Examples - Teiserver.Communication.RoomMessageLib.send_room_message/4","doc":"iex> send_room_message(123, 123, \"Message content\")\n      {:ok, %RoomMessage{}}\n\n      iex> send_room_message(456, 456, \"Message content\")\n      {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.RoomMessageLib.html#send_room_message/4-examples"},{"type":"function","title":"Teiserver.Communication.RoomMessageLib.subscribe_to_room_messages/1","doc":"Joins your process to room messages","ref":"Teiserver.Communication.RoomMessageLib.html#subscribe_to_room_messages/1"},{"type":"function","title":"Teiserver.Communication.RoomMessageLib.unsubscribe_from_room_messages/1","doc":"Removes your process from a room's messages","ref":"Teiserver.Communication.RoomMessageLib.html#unsubscribe_from_room_messages/1"},{"type":"function","title":"Teiserver.Communication.RoomMessageLib.update_room_message/2","doc":"Updates a room_message.","ref":"Teiserver.Communication.RoomMessageLib.html#update_room_message/2"},{"type":"function","title":"Examples - Teiserver.Communication.RoomMessageLib.update_room_message/2","doc":"iex> update_room_message(room_message, %{field: new_value})\n    {:ok, %RoomMessage{}}\n\n    iex> update_room_message(room_message, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Communication.RoomMessageLib.html#update_room_message/2-examples"},{"type":"module","title":"Teiserver.Connections.Client","doc":"# Client\nA client represents the logged in instance of a user. Clients are not persisted in the database, instead they are present only in the `Teiserver.Connections.ClientServer` processes.\n\n#","ref":"Teiserver.Connections.Client.html"},{"type":"module","title":"Attributes - Teiserver.Connections.Client","doc":"* `:id` - The user_id of the User being represented\n* `:connected?` - When true, at least one connection for the client is still live, when false no connections for the client are live\n* `:last_disconnected` - When disconnected, stores a DateTime of when the client was last connected\n* `:lobby_id` - nil or the id of the lobby current occupied by this client\n* `:in_game?` - True when the client is in-game\n* `:afk?` - True when the client has not sent an activity message for a while\n* `:ready?` - A flag set by the client to show it is ready to proceed in the current lobby\n* `:player?` - When in a lobby or match, set to true if the client is playing and false if not (e.g. spectator)\n* `:player_number` - The numerical ID of the player within a lobby or match\n* `:team_number` - The numerical ID of the team the player is present on within a lobby or match\n* `:team_colour` - The colour used to represent this client, it is a freeform string so you are able to extend and overload this as you see fit\n* `:sync` - A map used for storing keys of items the client needs to sync and the values representing their state of syncing\n* `:lobby_host?` - Set to true if the client is the host of the lobby in question\n* `:party_id` - The ID of the party the client is part of or nil if not in a party\n\n* `:update_id` - The version ID of the update to ensure updates can be discarded if out of date or duplicates","ref":"Teiserver.Connections.Client.html#module-attributes"},{"type":"function","title":"Teiserver.Connections.Client.new/1","doc":"","ref":"Teiserver.Connections.Client.html#new/1"},{"type":"type","title":"Teiserver.Connections.Client.t/0","doc":"","ref":"Teiserver.Connections.Client.html#t:t/0"},{"type":"module","title":"Teiserver.Connections.ClientLib","doc":"Library of client related functions.","ref":"Teiserver.Connections.ClientLib.html"},{"type":"function","title":"Teiserver.Connections.ClientLib.connect_user/1","doc":"Given a user_id, log them in. If the user already exists as a client then the existing\nclient is returned.\n\nThe calling process will be listed as a connection for the client\nthe client will monitor it for the purposes of tracking if the\ngiven client is still connected.","ref":"Teiserver.Connections.ClientLib.html#connect_user/1"},{"type":"function","title":"Teiserver.Connections.ClientLib.disconnect_user/1","doc":"","ref":"Teiserver.Connections.ClientLib.html#disconnect_user/1"},{"type":"function","title":"Teiserver.Connections.ClientLib.get_client/1","doc":"Gets a single client from a user_id.\n\nReturns nil if the Client does not exist.","ref":"Teiserver.Connections.ClientLib.html#get_client/1"},{"type":"function","title":"Examples - Teiserver.Connections.ClientLib.get_client/1","doc":"iex> get_client(123)\n    %Client{}\n\n    iex> get_client(456)\n    nil","ref":"Teiserver.Connections.ClientLib.html#get_client/1-examples"},{"type":"function","title":"Teiserver.Connections.ClientLib.get_client_list/1","doc":"Given a list of ids, return a list of client states.\n\nReturns nil if the Client does not exist.","ref":"Teiserver.Connections.ClientLib.html#get_client_list/1"},{"type":"function","title":"Examples - Teiserver.Connections.ClientLib.get_client_list/1","doc":"iex> get_client_list([123, 124])\n    [%Client{}, %Client{}]\n\n    iex> get_client_list([456])\n    [nil]","ref":"Teiserver.Connections.ClientLib.html#get_client_list/1-examples"},{"type":"function","title":"Teiserver.Connections.ClientLib.list_client_ids/0","doc":"Returns the list of client ids.","ref":"Teiserver.Connections.ClientLib.html#list_client_ids/0"},{"type":"function","title":"Examples - Teiserver.Connections.ClientLib.list_client_ids/0","doc":"iex> list_client_ids()\n    [123, ...]","ref":"Teiserver.Connections.ClientLib.html#list_client_ids/0-examples"},{"type":"function","title":"Teiserver.Connections.ClientLib.list_local_client_ids/0","doc":"","ref":"Teiserver.Connections.ClientLib.html#list_local_client_ids/0"},{"type":"function","title":"Teiserver.Connections.ClientLib.subscribe_to_client/1","doc":"Subscribes the process to client updates for this user","ref":"Teiserver.Connections.ClientLib.html#subscribe_to_client/1"},{"type":"function","title":"Teiserver.Connections.ClientLib.unsubscribe_from_client/1","doc":"Unsubscribes the process to client updates for this user","ref":"Teiserver.Connections.ClientLib.html#unsubscribe_from_client/1"},{"type":"function","title":"Teiserver.Connections.ClientLib.update_client/3","doc":"Updates a client with the new data (excluding lobby related details). If changes are made then it will also generate a `Teiserver.Connections.Client:{user_id}` pubsub message.\n\nReturns nil if the Client does not exist, :ok if the client does.","ref":"Teiserver.Connections.ClientLib.html#update_client/3"},{"type":"function","title":"Examples - Teiserver.Connections.ClientLib.update_client/3","doc":"iex> update_client(123, %{afk?: false})\n    :ok\n\n    iex> update_client(456, %{afk?: false})\n    nil","ref":"Teiserver.Connections.ClientLib.html#update_client/3-examples"},{"type":"function","title":"Teiserver.Connections.ClientLib.update_client_full/3","doc":"Updates a client with the new data for any and all keys (so be careful not to break things like lobby memberships).\n\nReturns nil if the Client does not exist, :ok if the client does.","ref":"Teiserver.Connections.ClientLib.html#update_client_full/3"},{"type":"function","title":"Examples - Teiserver.Connections.ClientLib.update_client_full/3","doc":"iex> update_client_full(123, %{player_number: 123})\n    :ok\n\n    iex> update_client_full(456, %{player_number: 123})\n    nil","ref":"Teiserver.Connections.ClientLib.html#update_client_full/3-examples"},{"type":"function","title":"Teiserver.Connections.ClientLib.update_client_in_lobby/3","doc":"Updates a client with the new data for their lobby presence. If changes are made then it will also generate a `Teiserver.Connections.Client:{user_id}` pubsub message and a `Teiserver.Game.Lobby:{lobby_id}` pubsub message.\n\nReturns nil if the Client does not exist, :ok if the client does.","ref":"Teiserver.Connections.ClientLib.html#update_client_in_lobby/3"},{"type":"function","title":"Examples - Teiserver.Connections.ClientLib.update_client_in_lobby/3","doc":"iex> update_client_in_lobby(123, %{player_number: 123})\n    :ok\n\n    iex> update_client_in_lobby(456, %{player_number: 123})\n    nil","ref":"Teiserver.Connections.ClientLib.html#update_client_in_lobby/3-examples"},{"type":"module","title":"Teiserver.Connections.ClientServer","doc":"A process representing the state of a client. In the normal usage of Teiserver\nyou are not expected to interact with it directly.","ref":"Teiserver.Connections.ClientServer.html"},{"type":"function","title":"Teiserver.Connections.ClientServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Teiserver.Connections.ClientServer.html#child_spec/1"},{"type":"module","title":"Teiserver.Game.Lobby","doc":"# Lobby\nA lobby is a place for users to gather and form a game instance. Once the lobby starts it becomes a match, once the match ends the users return to the lobby.\n\nFor a guide on the lifecycle of lobbies and matches please see the [`Match lifecycle guide`](match_lifecycle.html).\n\n#","ref":"Teiserver.Game.Lobby.html"},{"type":"module","title":"Attributes - Teiserver.Game.Lobby","doc":"* `:id` - The id of the lobby in question\n* `:match_id` - The id of the match for this lobby\n* `:host_id` - The user_id of the creator of the lobby\n* `:match_ongoing?` - True if the match is currently in progress\n* `:host_data` - A map of the data relevant to the host of the game, if the data is set to nil it means the host is currently disconnected but the ClientServer process has yet to time out\n* `:name` - The name of the lobby as displayed in lobby lists\n* `:tags` - A list of strings representing tagged values\n* `:password` - The (plaintext) password for the lobby, nil if no password\n* `:passworded?` - Boolean of the room having a password or not, overloads the :password property for ease of use\n* `:locked?` - Boolean of the room being locked to the general public at this stage\n* `:public?` - Boolean of the room being public or not, when set to false updates to this lobby won't appear in global_battle updates\n* `:match_type` - MatchType.id of the type of match this will be\n* `:rated?` - When set to true it means the game will be rated (assuming all other requirements are met)\n* `:queue_id` - The ID of the queue (matchmaking) this lobby belongs to\n* `:game_name` - String of the game name\n* `:game_version` - String of the game version\n* `:game_settings` - Map of the settings to be used in starting the game\n* `:players` - List of user_ids as players, source of truth is still client state\n* `:spectators` - List of user_ids spectating, source of truth is still client state\n* `:members` - Total list of all user_ids who are members of the lobby\n* `:approved_members` - A list of users approved to join the lobby bypassing some of the normal checks\n\n* `:update_id` - The version ID of the update to ensure updates can be discarded if out of date or duplicates","ref":"Teiserver.Game.Lobby.html#module-attributes"},{"type":"function","title":"Teiserver.Game.Lobby.new/2","doc":"","ref":"Teiserver.Game.Lobby.html#new/2"},{"type":"type","title":"Teiserver.Game.Lobby.id/0","doc":"","ref":"Teiserver.Game.Lobby.html#t:id/0"},{"type":"type","title":"Teiserver.Game.Lobby.name/0","doc":"","ref":"Teiserver.Game.Lobby.html#t:name/0"},{"type":"type","title":"Teiserver.Game.Lobby.t/0","doc":"A lobby","ref":"Teiserver.Game.Lobby.html#t:t/0"},{"type":"module","title":"Teiserver.Game.LobbyLib","doc":"TODO: Library of lobby related functions.","ref":"Teiserver.Game.LobbyLib.html"},{"type":"function","title":"Teiserver.Game.LobbyLib.add_client_to_lobby/2","doc":"Adds a client to the lobby","ref":"Teiserver.Game.LobbyLib.html#add_client_to_lobby/2"},{"type":"function","title":"Teiserver.Game.LobbyLib.can_add_client_to_lobby/3","doc":"Adds a client to the lobby","ref":"Teiserver.Game.LobbyLib.html#can_add_client_to_lobby/3"},{"type":"function","title":"Teiserver.Game.LobbyLib.client_update_request/2","doc":"Used to tell a lobby process the current match has started","ref":"Teiserver.Game.LobbyLib.html#client_update_request/2"},{"type":"function","title":"Examples - Teiserver.Game.LobbyLib.client_update_request/2","doc":"iex> client_update_request(%{team_number: 1, id: 1}, 123)\n    %{}\n\n    iex> client_update_request(%{team_number: 1, id: 456}, 456)\n    nil","ref":"Teiserver.Game.LobbyLib.html#client_update_request/2-examples"},{"type":"function","title":"Teiserver.Game.LobbyLib.close_lobby/1","doc":"Given a lobby_id it will close the lobby. Every client in the lobby will be\nremoved from the lobby.","ref":"Teiserver.Game.LobbyLib.html#close_lobby/1"},{"type":"function","title":"Examples - Teiserver.Game.LobbyLib.close_lobby/1","doc":"iex> close_lobby(123)\n  :ok","ref":"Teiserver.Game.LobbyLib.html#close_lobby/1-examples"},{"type":"function","title":"Teiserver.Game.LobbyLib.cycle_lobby/1","doc":"Used to cycle a lobby after a match has concluded.","ref":"Teiserver.Game.LobbyLib.html#cycle_lobby/1"},{"type":"function","title":"Examples - Teiserver.Game.LobbyLib.cycle_lobby/1","doc":"iex> cycle_lobby(123)\n    :ok\n\n    iex> cycle_lobby(456)\n    nil","ref":"Teiserver.Game.LobbyLib.html#cycle_lobby/1-examples"},{"type":"function","title":"Teiserver.Game.LobbyLib.default_lobby_name_acceptable?/1","doc":"","ref":"Teiserver.Game.LobbyLib.html#default_lobby_name_acceptable?/1"},{"type":"function","title":"Teiserver.Game.LobbyLib.get_lobby/1","doc":"","ref":"Teiserver.Game.LobbyLib.html#get_lobby/1"},{"type":"function","title":"Teiserver.Game.LobbyLib.get_lobby_attribute/2","doc":"","ref":"Teiserver.Game.LobbyLib.html#get_lobby_attribute/2"},{"type":"function","title":"Teiserver.Game.LobbyLib.get_lobby_summary/1","doc":"","ref":"Teiserver.Game.LobbyLib.html#get_lobby_summary/1"},{"type":"function","title":"Teiserver.Game.LobbyLib.list_lobby_ids/0","doc":"","ref":"Teiserver.Game.LobbyLib.html#list_lobby_ids/0"},{"type":"function","title":"Teiserver.Game.LobbyLib.list_local_lobby_ids/0","doc":"","ref":"Teiserver.Game.LobbyLib.html#list_local_lobby_ids/0"},{"type":"function","title":"Teiserver.Game.LobbyLib.lobby_name_acceptable?/1","doc":"Tests is the lobby name is acceptable. Can be over-ridden using the config [fn_lobby_name_acceptor](config.html#fn_lobby_name_acceptor)","ref":"Teiserver.Game.LobbyLib.html#lobby_name_acceptable?/1"},{"type":"function","title":"Teiserver.Game.LobbyLib.lobby_start_match/1","doc":"Used to tell a lobby process the current match has started","ref":"Teiserver.Game.LobbyLib.html#lobby_start_match/1"},{"type":"function","title":"Examples - Teiserver.Game.LobbyLib.lobby_start_match/1","doc":"iex> lobby_start_match(123)\n    :ok\n\n    iex> lobby_start_match(456)\n    nil","ref":"Teiserver.Game.LobbyLib.html#lobby_start_match/1-examples"},{"type":"function","title":"Teiserver.Game.LobbyLib.open_lobby/2","doc":"Given a user_id of the host and the initial lobby name, starts a process\nfor tracking the lobby.\n\nIt will then update the client of the host user and assign them to the lobby.","ref":"Teiserver.Game.LobbyLib.html#open_lobby/2"},{"type":"function","title":"Examples - Teiserver.Game.LobbyLib.open_lobby/2","doc":"iex> open_lobby(123, \"Name\")\n    {:ok, 456}\n\n    iex> open_lobby(456, \"Name\")\n    {:error, \"Client is not connected\"}","ref":"Teiserver.Game.LobbyLib.html#open_lobby/2-examples"},{"type":"function","title":"Teiserver.Game.LobbyLib.remove_client_from_lobby/2","doc":"Adds a client to the lobby","ref":"Teiserver.Game.LobbyLib.html#remove_client_from_lobby/2"},{"type":"function","title":"Teiserver.Game.LobbyLib.stream_lobby_summaries/0","doc":"","ref":"Teiserver.Game.LobbyLib.html#stream_lobby_summaries/0"},{"type":"function","title":"Teiserver.Game.LobbyLib.stream_lobby_summaries/1","doc":"","ref":"Teiserver.Game.LobbyLib.html#stream_lobby_summaries/1"},{"type":"function","title":"Teiserver.Game.LobbyLib.subscribe_to_lobby/1","doc":"Subscribes the process to lobby updates for this user","ref":"Teiserver.Game.LobbyLib.html#subscribe_to_lobby/1"},{"type":"function","title":"Teiserver.Game.LobbyLib.unsubscribe_from_lobby/1","doc":"Unsubscribes the process to lobby updates for this user","ref":"Teiserver.Game.LobbyLib.html#unsubscribe_from_lobby/1"},{"type":"function","title":"Teiserver.Game.LobbyLib.update_lobby/2","doc":"","ref":"Teiserver.Game.LobbyLib.html#update_lobby/2"},{"type":"module","title":"Teiserver.Game.LobbyServer","doc":"A process representing the state of a lobby.\nIn the normal usage of Teiserver you are not expected to interact with it directly.","ref":"Teiserver.Game.LobbyServer.html"},{"type":"function","title":"Teiserver.Game.LobbyServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Teiserver.Game.LobbyServer.html#child_spec/1"},{"type":"function","title":"Teiserver.Game.LobbyServer.update_lobby/2","doc":"","ref":"Teiserver.Game.LobbyServer.html#update_lobby/2"},{"type":"module","title":"Teiserver.Game.LobbySummary","doc":"# LobbySummary\nA type used to represent only part of a lobby (e.g. for lists of lobbies)\n\n#","ref":"Teiserver.Game.LobbySummary.html"},{"type":"module","title":"Attributes - Teiserver.Game.LobbySummary","doc":"* `:id` - The id of the lobby in question\n* `:host_id` - The user_id of the creator of the lobby\n* `:match_ongoing?` - True if the match is currently in progress\n* `:name` - The name of the lobby as displayed in lobby lists\n* `:tags` - A list of strings representing tagged values\n* `:password` - The (plaintext) password for the lobby, nil if no password\n* `:locked?` - Boolean of the room being locked to the general public at this stage\n* `:public?` - Boolean of the room being public or not, when set to false updates to this lobby won't appear in global_battle updates\n* `:rated?` - When set to true it means the game will be rated (assuming all other requirements are met)\n* `:queue_id` - The ID of the queue (matchmaking) this lobby belongs to\n* `:game_name` - String of the game name\n* `:game_version` - String of the game version\n* `:player_count` - Count of the number of players\n* `:spectator_count` - Count of the number of spectators","ref":"Teiserver.Game.LobbySummary.html#module-attributes"},{"type":"function","title":"Teiserver.Game.LobbySummary.new/1","doc":"","ref":"Teiserver.Game.LobbySummary.html#new/1"},{"type":"type","title":"Teiserver.Game.LobbySummary.t/0","doc":"A lobby","ref":"Teiserver.Game.LobbySummary.html#t:t/0"},{"type":"module","title":"Teiserver.Game.Match","doc":"# Match\nThe persisted storage of a game, for a guide on the lifecycle of a match please see the [`Match lifecycle guide`](match_lifecycle.html).\n\n#","ref":"Teiserver.Game.Match.html"},{"type":"module","title":"Attributes - Teiserver.Game.Match","doc":"* `:name` - The name of the lobby at the time the match started\n* `:tags` - The lobby tags at the time of the match starting\n* `:public?` - The public flag at time of the match starting\n* `:rated?` - The rated flag at time of the match starting, updated to false if the match is later decided to be unrated\n* `:game_name` - The game_name at time of the match starting, in most cases I expect game name to always be the same but this is in place to allow for multiple games to be run from a single server\n* `:game_version` - The game_version at time of the match starting\n* `:winning_team` - The ID of the winning team, set to nil in a draw\n* `:team_count` - The number of teams in the game\n* `:team_size` - The size of the largest team\n* `:processed?` - A flag showing if the match has had post-match processing\n* `:game_type` - A string indicating the type of game\n* `:lobby_opened_at` - The datetime the lobby was opened at or when the previous match completed\n* `:match_started_at` - The datetime the lobby swapped to being in progress, not the actual start time of the game itself\n* `:match_ended_at` - The datetime the match ended and the players were returned to the lobby, not the actual end time of the game itself\n* `:ended_normally?` - True if the match was ended in a normal manner, false if ended in an abnormal manner (e.g. crashed)\n* `:match_duration_seconds` - The duration of the game in seconds as indicated by the game host\n* `:host` - The user account hosting the lobby","ref":"Teiserver.Game.Match.html#module-attributes"},{"type":"function","title":"Teiserver.Game.Match.changeset/2","doc":"Builds a changeset based on the `struct` and `params`.","ref":"Teiserver.Game.Match.html#changeset/2"},{"type":"type","title":"Teiserver.Game.Match.id/0","doc":"","ref":"Teiserver.Game.Match.html#t:id/0"},{"type":"type","title":"Teiserver.Game.Match.t/0","doc":"","ref":"Teiserver.Game.Match.html#t:t/0"},{"type":"module","title":"Teiserver.Game.MatchLib","doc":"TODO: Library of match related functions.","ref":"Teiserver.Game.MatchLib.html"},{"type":"function","title":"Teiserver.Game.MatchLib.change_match/2","doc":"Returns an `%Ecto.Changeset{}` for tracking match changes.","ref":"Teiserver.Game.MatchLib.html#change_match/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchLib.change_match/2","doc":"iex> change_match(match)\n    %Ecto.Changeset{data: %Match{}}","ref":"Teiserver.Game.MatchLib.html#change_match/2-examples"},{"type":"function","title":"Teiserver.Game.MatchLib.create_match/1","doc":"Creates a match.","ref":"Teiserver.Game.MatchLib.html#create_match/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchLib.create_match/1","doc":"iex> create_match(%{field: value})\n    {:ok, %Match{}}\n\n    iex> create_match(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchLib.html#create_match/1-examples"},{"type":"function","title":"Teiserver.Game.MatchLib.delete_match/1","doc":"Deletes a match.","ref":"Teiserver.Game.MatchLib.html#delete_match/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchLib.delete_match/1","doc":"iex> delete_match(match)\n    {:ok, %Match{}}\n\n    iex> delete_match(match)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchLib.html#delete_match/1-examples"},{"type":"function","title":"Teiserver.Game.MatchLib.end_match/2","doc":"Ends an ongoing match and updates memberships accordingly.\n\nSecond argument is a map of the outcomes for the match with the following keys:\n* `:winning_team` - The team_number of the winning team\n* `:ended_normally?` - A boolean indicating if the match ended normally\n* `:players` - A map of player data with the keys being the user_id of the player and the value being a map of their specific outcome\n\nPlayer outcomes are expected to map like so:\nRequired:\n\nOptional:\n* `:left_after_seconds` - The number of seconds after the start the player left if early. If not included it is assumed the player remained until the end.","ref":"Teiserver.Game.MatchLib.html#end_match/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchLib.end_match/2","doc":"iex> end_match(123, %{winning_team: 1, ended_normally?: true, player_data: %{123: }})\n    %Match{}","ref":"Teiserver.Game.MatchLib.html#end_match/2-examples"},{"type":"function","title":"Teiserver.Game.MatchLib.get_match/2","doc":"Gets a single match.\n\nReturns nil if the Match does not exist.","ref":"Teiserver.Game.MatchLib.html#get_match/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchLib.get_match/2","doc":"iex> get_match(123)\n    %Match{}\n\n    iex> get_match(456)\n    nil","ref":"Teiserver.Game.MatchLib.html#get_match/2-examples"},{"type":"function","title":"Teiserver.Game.MatchLib.get_match!/2","doc":"Gets a single match.\n\nRaises `Ecto.NoResultsError` if the Match does not exist.","ref":"Teiserver.Game.MatchLib.html#get_match!/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchLib.get_match!/2","doc":"iex> get_match!(123)\n    %Match{}\n\n    iex> get_match!(456)\n    ** (Ecto.NoResultsError)","ref":"Teiserver.Game.MatchLib.html#get_match!/2-examples"},{"type":"function","title":"Teiserver.Game.MatchLib.list_matches/1","doc":"Returns the list of matches.","ref":"Teiserver.Game.MatchLib.html#list_matches/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchLib.list_matches/1","doc":"iex> list_matches()\n    [%Match{}, ...]","ref":"Teiserver.Game.MatchLib.html#list_matches/1-examples"},{"type":"function","title":"Teiserver.Game.MatchLib.start_match/1","doc":"Given a lobby_id, will update the match, memberships and settings for that lobby\nand then update the Lobby itself to show the lobby is now in progress.","ref":"Teiserver.Game.MatchLib.html#start_match/1"},{"type":"function","title":"Teiserver.Game.MatchLib.update_match/2","doc":"Updates a match.","ref":"Teiserver.Game.MatchLib.html#update_match/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchLib.update_match/2","doc":"iex> update_match(match, %{field: new_value})\n    {:ok, %Match{}}\n\n    iex> update_match(match, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchLib.html#update_match/2-examples"},{"type":"module","title":"Teiserver.Game.MatchMembership","doc":"# Match\nThe persisted storage of players of a match after it started.\n\n#","ref":"Teiserver.Game.MatchMembership.html"},{"type":"module","title":"Attributes - Teiserver.Game.MatchMembership","doc":"* `:team_number` - The team this player was a member of\n* `:win?` - If the player was on the winning team\n* `:party_id` - The ID of the party the player was a member of (if any)\n* `:left_after_seconds` - The number of seconds into the game at which the player was determined to have disconnected\n* `:user` - The user in question\n* `:match` - The match in question","ref":"Teiserver.Game.MatchMembership.html#module-attributes"},{"type":"function","title":"Teiserver.Game.MatchMembership.changeset/2","doc":"Builds a changeset based on the `struct` and `params`.","ref":"Teiserver.Game.MatchMembership.html#changeset/2"},{"type":"type","title":"Teiserver.Game.MatchMembership.t/0","doc":"","ref":"Teiserver.Game.MatchMembership.html#t:t/0"},{"type":"module","title":"Teiserver.Game.MatchMembershipLib","doc":"TODO: Library of match_membership related functions.","ref":"Teiserver.Game.MatchMembershipLib.html"},{"type":"function","title":"Teiserver.Game.MatchMembershipLib.change_match_membership/2","doc":"Returns an `%Ecto.Changeset{}` for tracking match_membership changes.","ref":"Teiserver.Game.MatchMembershipLib.html#change_match_membership/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchMembershipLib.change_match_membership/2","doc":"iex> change_match_membership(match_membership)\n    %Ecto.Changeset{data: %MatchMembership{}}","ref":"Teiserver.Game.MatchMembershipLib.html#change_match_membership/2-examples"},{"type":"function","title":"Teiserver.Game.MatchMembershipLib.create_many_match_memberships/1","doc":"Creates many match_memberships. Not unlike most other create functions this will raise an exception on failure and should not be caught using the normal case functions.\n\nExpects a map of values which can be turned into valid match memberships.","ref":"Teiserver.Game.MatchMembershipLib.html#create_many_match_memberships/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchMembershipLib.create_many_match_memberships/1","doc":"iex> create_many_match_memberships([%{field: value}])\n    {:ok, %MatchMembership{}}\n\n    iex> create_many_match_memberships([%{field: bad_value}])\n    raise Postgrex.Error","ref":"Teiserver.Game.MatchMembershipLib.html#create_many_match_memberships/1-examples"},{"type":"function","title":"Teiserver.Game.MatchMembershipLib.create_match_membership/1","doc":"Creates a match_membership.","ref":"Teiserver.Game.MatchMembershipLib.html#create_match_membership/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchMembershipLib.create_match_membership/1","doc":"iex> create_match_membership(%{field: value})\n    {:ok, %MatchMembership{}}\n\n    iex> create_match_membership(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchMembershipLib.html#create_match_membership/1-examples"},{"type":"function","title":"Teiserver.Game.MatchMembershipLib.delete_match_membership/1","doc":"Deletes a match_membership.","ref":"Teiserver.Game.MatchMembershipLib.html#delete_match_membership/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchMembershipLib.delete_match_membership/1","doc":"iex> delete_match_membership(match_membership)\n    {:ok, %MatchMembership{}}\n\n    iex> delete_match_membership(match_membership)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchMembershipLib.html#delete_match_membership/1-examples"},{"type":"function","title":"Teiserver.Game.MatchMembershipLib.get_match_membership/3","doc":"Gets a single match_membership.\n\nReturns nil if the MatchMembership does not exist.","ref":"Teiserver.Game.MatchMembershipLib.html#get_match_membership/3"},{"type":"function","title":"Examples - Teiserver.Game.MatchMembershipLib.get_match_membership/3","doc":"iex> get_match_membership(123)\n    %MatchMembership{}\n\n    iex> get_match_membership(456)\n    nil","ref":"Teiserver.Game.MatchMembershipLib.html#get_match_membership/3-examples"},{"type":"function","title":"Teiserver.Game.MatchMembershipLib.get_match_membership!/3","doc":"Gets a single match_membership.\n\nRaises `Ecto.NoResultsError` if the MatchMembership does not exist.","ref":"Teiserver.Game.MatchMembershipLib.html#get_match_membership!/3"},{"type":"function","title":"Examples - Teiserver.Game.MatchMembershipLib.get_match_membership!/3","doc":"iex> get_match_membership!(123)\n    %MatchMembership{}\n\n    iex> get_match_membership!(456)\n    ** (Ecto.NoResultsError)","ref":"Teiserver.Game.MatchMembershipLib.html#get_match_membership!/3-examples"},{"type":"function","title":"Teiserver.Game.MatchMembershipLib.list_match_memberships/1","doc":"Returns the list of match_memberships.","ref":"Teiserver.Game.MatchMembershipLib.html#list_match_memberships/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchMembershipLib.list_match_memberships/1","doc":"iex> list_match_memberships()\n    [%MatchMembership{}, ...]","ref":"Teiserver.Game.MatchMembershipLib.html#list_match_memberships/1-examples"},{"type":"function","title":"Teiserver.Game.MatchMembershipLib.update_match_membership/2","doc":"Updates a match_membership.","ref":"Teiserver.Game.MatchMembershipLib.html#update_match_membership/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchMembershipLib.update_match_membership/2","doc":"iex> update_match_membership(match_membership, %{field: new_value})\n    {:ok, %MatchMembership{}}\n\n    iex> update_match_membership(match_membership, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchMembershipLib.html#update_match_membership/2-examples"},{"type":"module","title":"Teiserver.Game.MatchSetting","doc":"# MatchSetting\nSettings applied to matches\n\n#","ref":"Teiserver.Game.MatchSetting.html"},{"type":"module","title":"Attributes - Teiserver.Game.MatchSetting","doc":"* `:type_id`/`:type` - The type of setting\n* `:match_id`/`:match` - The match the setting was used for\n* `:value` - The value of the setting","ref":"Teiserver.Game.MatchSetting.html#module-attributes"},{"type":"function","title":"Teiserver.Game.MatchSetting.changeset/2","doc":"Builds a changeset based on the `struct` and `params`.","ref":"Teiserver.Game.MatchSetting.html#changeset/2"},{"type":"type","title":"Teiserver.Game.MatchSetting.t/0","doc":"","ref":"Teiserver.Game.MatchSetting.html#t:t/0"},{"type":"module","title":"Teiserver.Game.MatchSettingLib","doc":"TODO: Library of match_setting related functions.","ref":"Teiserver.Game.MatchSettingLib.html"},{"type":"function","title":"Teiserver.Game.MatchSettingLib.change_match_setting/2","doc":"Returns an `%Ecto.Changeset{}` for tracking match_setting changes.","ref":"Teiserver.Game.MatchSettingLib.html#change_match_setting/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingLib.change_match_setting/2","doc":"iex> change_match_setting(match_setting)\n    %Ecto.Changeset{data: %MatchSetting{}}","ref":"Teiserver.Game.MatchSettingLib.html#change_match_setting/2-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingLib.create_many_match_settings/1","doc":"Creates many match_settings. Not unlike most other create functions this will raise an exception on failure and should not be caught using the normal case functions.\n\nExpects a map of values which can be turned into valid match settings.","ref":"Teiserver.Game.MatchSettingLib.html#create_many_match_settings/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingLib.create_many_match_settings/1","doc":"iex> create_many_match_settings([%{field: value}])\n    {:ok, %MatchSetting{}}\n\n    iex> create_many_match_settings([%{field: bad_value}])\n    raise Postgrex.Error","ref":"Teiserver.Game.MatchSettingLib.html#create_many_match_settings/1-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingLib.create_match_setting/1","doc":"Creates a match_setting.","ref":"Teiserver.Game.MatchSettingLib.html#create_match_setting/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingLib.create_match_setting/1","doc":"iex> create_match_setting(%{field: value})\n    {:ok, %MatchSetting{}}\n\n    iex> create_match_setting(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchSettingLib.html#create_match_setting/1-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingLib.delete_match_setting/1","doc":"Deletes a match_setting.","ref":"Teiserver.Game.MatchSettingLib.html#delete_match_setting/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingLib.delete_match_setting/1","doc":"iex> delete_match_setting(match_setting)\n    {:ok, %MatchSetting{}}\n\n    iex> delete_match_setting(match_setting)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchSettingLib.html#delete_match_setting/1-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingLib.get_match_setting/3","doc":"Gets a single match_setting.\n\nReturns nil if the MatchSetting does not exist.","ref":"Teiserver.Game.MatchSettingLib.html#get_match_setting/3"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingLib.get_match_setting/3","doc":"iex> get_match_setting(123)\n    %MatchSetting{}\n\n    iex> get_match_setting(456)\n    nil","ref":"Teiserver.Game.MatchSettingLib.html#get_match_setting/3-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingLib.get_match_setting!/3","doc":"Gets a single match_setting.\n\nRaises `Ecto.NoResultsError` if the MatchSetting does not exist.","ref":"Teiserver.Game.MatchSettingLib.html#get_match_setting!/3"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingLib.get_match_setting!/3","doc":"iex> get_match_setting!(123)\n    %MatchSetting{}\n\n    iex> get_match_setting!(456)\n    ** (Ecto.NoResultsError)","ref":"Teiserver.Game.MatchSettingLib.html#get_match_setting!/3-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingLib.get_match_settings_map/1","doc":"Returns a key => value map of match settings for a given match_id.","ref":"Teiserver.Game.MatchSettingLib.html#get_match_settings_map/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingLib.get_match_settings_map/1","doc":"iex> get_match_settings_map(123)\n    %{\"key1\" => \"value1\", \"key2\" => \"value2\"}\n\n    iex> get_match_settings_map(456)\n    %{}","ref":"Teiserver.Game.MatchSettingLib.html#get_match_settings_map/1-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingLib.list_match_settings/1","doc":"Returns the list of match_settings.","ref":"Teiserver.Game.MatchSettingLib.html#list_match_settings/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingLib.list_match_settings/1","doc":"iex> list_match_settings()\n    [%MatchSetting{}, ...]","ref":"Teiserver.Game.MatchSettingLib.html#list_match_settings/1-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingLib.update_match_setting/2","doc":"Updates a match_setting.","ref":"Teiserver.Game.MatchSettingLib.html#update_match_setting/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingLib.update_match_setting/2","doc":"iex> update_match_setting(match_setting, %{field: new_value})\n    {:ok, %MatchSetting{}}\n\n    iex> update_match_setting(match_setting, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchSettingLib.html#update_match_setting/2-examples"},{"type":"module","title":"Teiserver.Game.MatchSettingType","doc":"# MatchSettingType\nType information regarding match setting values\n\n#","ref":"Teiserver.Game.MatchSettingType.html"},{"type":"module","title":"Attributes - Teiserver.Game.MatchSettingType","doc":"* `:name` - The name of the setting","ref":"Teiserver.Game.MatchSettingType.html#module-attributes"},{"type":"function","title":"Teiserver.Game.MatchSettingType.changeset/2","doc":"Builds a changeset based on the `struct` and `params`.","ref":"Teiserver.Game.MatchSettingType.html#changeset/2"},{"type":"type","title":"Teiserver.Game.MatchSettingType.id/0","doc":"","ref":"Teiserver.Game.MatchSettingType.html#t:id/0"},{"type":"type","title":"Teiserver.Game.MatchSettingType.t/0","doc":"","ref":"Teiserver.Game.MatchSettingType.html#t:t/0"},{"type":"module","title":"Teiserver.Game.MatchSettingTypeLib","doc":"TODO: Library of match_setting_type related functions.","ref":"Teiserver.Game.MatchSettingTypeLib.html"},{"type":"function","title":"Teiserver.Game.MatchSettingTypeLib.change_match_setting_type/2","doc":"Returns an `%Ecto.Changeset{}` for tracking match_setting_type changes.","ref":"Teiserver.Game.MatchSettingTypeLib.html#change_match_setting_type/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingTypeLib.change_match_setting_type/2","doc":"iex> change_match_setting_type(match_setting_type)\n    %Ecto.Changeset{data: %MatchSettingType{}}","ref":"Teiserver.Game.MatchSettingTypeLib.html#change_match_setting_type/2-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingTypeLib.create_match_setting_type/1","doc":"Creates a match_setting_type.","ref":"Teiserver.Game.MatchSettingTypeLib.html#create_match_setting_type/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingTypeLib.create_match_setting_type/1","doc":"iex> create_match_setting_type(%{field: value})\n    {:ok, %MatchSettingType{}}\n\n    iex> create_match_setting_type(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchSettingTypeLib.html#create_match_setting_type/1-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingTypeLib.delete_match_setting_type/1","doc":"Deletes a match_setting_type.","ref":"Teiserver.Game.MatchSettingTypeLib.html#delete_match_setting_type/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingTypeLib.delete_match_setting_type/1","doc":"iex> delete_match_setting_type(match_setting_type)\n    {:ok, %MatchSettingType{}}\n\n    iex> delete_match_setting_type(match_setting_type)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchSettingTypeLib.html#delete_match_setting_type/1-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingTypeLib.get_match_setting_type/2","doc":"Gets a single match_setting_type.\n\nReturns nil if the MatchSettingType does not exist.","ref":"Teiserver.Game.MatchSettingTypeLib.html#get_match_setting_type/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingTypeLib.get_match_setting_type/2","doc":"iex> get_match_setting_type(123)\n    %MatchSettingType{}\n\n    iex> get_match_setting_type(456)\n    nil","ref":"Teiserver.Game.MatchSettingTypeLib.html#get_match_setting_type/2-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingTypeLib.get_match_setting_type!/2","doc":"Gets a single match_setting_type.\n\nRaises `Ecto.NoResultsError` if the MatchSettingType does not exist.","ref":"Teiserver.Game.MatchSettingTypeLib.html#get_match_setting_type!/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingTypeLib.get_match_setting_type!/2","doc":"iex> get_match_setting_type!(123)\n    %MatchSettingType{}\n\n    iex> get_match_setting_type!(456)\n    ** (Ecto.NoResultsError)","ref":"Teiserver.Game.MatchSettingTypeLib.html#get_match_setting_type!/2-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingTypeLib.get_or_create_match_setting_type/1","doc":"Gets the ID of a match_setting_type, if the type doesn't exist\nit will create the match_setting_type and return the id of that type","ref":"Teiserver.Game.MatchSettingTypeLib.html#get_or_create_match_setting_type/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingTypeLib.get_or_create_match_setting_type/1","doc":"iex> get_or_create_match_setting_type(\"existing\")\n    123\n\n    iex> get_or_create_match_setting_type(\"non-existing\")\n    1234","ref":"Teiserver.Game.MatchSettingTypeLib.html#get_or_create_match_setting_type/1-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingTypeLib.list_match_setting_types/1","doc":"Returns the list of match_setting_types.","ref":"Teiserver.Game.MatchSettingTypeLib.html#list_match_setting_types/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingTypeLib.list_match_setting_types/1","doc":"iex> list_match_setting_types()\n    [%MatchSettingType{}, ...]","ref":"Teiserver.Game.MatchSettingTypeLib.html#list_match_setting_types/1-examples"},{"type":"function","title":"Teiserver.Game.MatchSettingTypeLib.update_match_setting_type/2","doc":"Updates a match_setting_type.","ref":"Teiserver.Game.MatchSettingTypeLib.html#update_match_setting_type/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchSettingTypeLib.update_match_setting_type/2","doc":"iex> update_match_setting_type(match_setting_type, %{field: new_value})\n    {:ok, %MatchSettingType{}}\n\n    iex> update_match_setting_type(match_setting_type, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchSettingTypeLib.html#update_match_setting_type/2-examples"},{"type":"module","title":"Teiserver.Game.MatchType","doc":"# MatchType\nDetails for each type of match\n\n#","ref":"Teiserver.Game.MatchType.html"},{"type":"module","title":"Attributes - Teiserver.Game.MatchType","doc":"* `:name` - The name of the type","ref":"Teiserver.Game.MatchType.html#module-attributes"},{"type":"function","title":"Teiserver.Game.MatchType.changeset/2","doc":"Builds a changeset based on the `struct` and `params`.","ref":"Teiserver.Game.MatchType.html#changeset/2"},{"type":"type","title":"Teiserver.Game.MatchType.id/0","doc":"","ref":"Teiserver.Game.MatchType.html#t:id/0"},{"type":"type","title":"Teiserver.Game.MatchType.t/0","doc":"","ref":"Teiserver.Game.MatchType.html#t:t/0"},{"type":"module","title":"Teiserver.Game.MatchTypeLib","doc":"Library of match_type related functions.","ref":"Teiserver.Game.MatchTypeLib.html"},{"type":"function","title":"Teiserver.Game.MatchTypeLib.calculate_match_type/1","doc":"","ref":"Teiserver.Game.MatchTypeLib.html#calculate_match_type/1"},{"type":"function","title":"Teiserver.Game.MatchTypeLib.change_match_type/2","doc":"Returns an `%Ecto.Changeset{}` for tracking match_type changes.","ref":"Teiserver.Game.MatchTypeLib.html#change_match_type/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchTypeLib.change_match_type/2","doc":"iex> change_match_type(match_type)\n    %Ecto.Changeset{data: %MatchType{}}","ref":"Teiserver.Game.MatchTypeLib.html#change_match_type/2-examples"},{"type":"function","title":"Teiserver.Game.MatchTypeLib.create_match_type/1","doc":"Creates a match_type.","ref":"Teiserver.Game.MatchTypeLib.html#create_match_type/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchTypeLib.create_match_type/1","doc":"iex> create_match_type(%{field: value})\n    {:ok, %MatchType{}}\n\n    iex> create_match_type(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchTypeLib.html#create_match_type/1-examples"},{"type":"function","title":"Teiserver.Game.MatchTypeLib.default_calculate_match_type/1","doc":"The default method used to define match types\n\nCan be over-ridden using the config [fn_calculate_match_type](config.html#fn_calculate_match_type)","ref":"Teiserver.Game.MatchTypeLib.html#default_calculate_match_type/1"},{"type":"function","title":"Teiserver.Game.MatchTypeLib.delete_match_type/1","doc":"Deletes a match_type.","ref":"Teiserver.Game.MatchTypeLib.html#delete_match_type/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchTypeLib.delete_match_type/1","doc":"iex> delete_match_type(match_type)\n    {:ok, %MatchType{}}\n\n    iex> delete_match_type(match_type)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchTypeLib.html#delete_match_type/1-examples"},{"type":"function","title":"Teiserver.Game.MatchTypeLib.get_match_type/2","doc":"Gets a single match_type.\n\nReturns nil if the MatchType does not exist.","ref":"Teiserver.Game.MatchTypeLib.html#get_match_type/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchTypeLib.get_match_type/2","doc":"iex> get_match_type(123)\n    %MatchType{}\n\n    iex> get_match_type(456)\n    nil","ref":"Teiserver.Game.MatchTypeLib.html#get_match_type/2-examples"},{"type":"function","title":"Teiserver.Game.MatchTypeLib.get_match_type!/2","doc":"Gets a single match_type.\n\nRaises `Ecto.NoResultsError` if the MatchType does not exist.","ref":"Teiserver.Game.MatchTypeLib.html#get_match_type!/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchTypeLib.get_match_type!/2","doc":"iex> get_match_type!(123)\n    %MatchType{}\n\n    iex> get_match_type!(456)\n    ** (Ecto.NoResultsError)","ref":"Teiserver.Game.MatchTypeLib.html#get_match_type!/2-examples"},{"type":"function","title":"Teiserver.Game.MatchTypeLib.get_match_type_by_name_or_id/1","doc":"Gets a single match_type by name or id (must be an integer).\n\nReturns nil if the MatchType does not exist.","ref":"Teiserver.Game.MatchTypeLib.html#get_match_type_by_name_or_id/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchTypeLib.get_match_type_by_name_or_id/1","doc":"iex> get_match_type_by_name_or_id(\"main\")\n    %MatchType{}\n\n    iex> get_match_type_by_name_or_id(123)\n    %MatchType{}\n\n    iex> get_match_type_by_name_or_id(\"not a name\")\n    nil\n\n    iex> get_match_type_by_name_or_id(456)\n    nil","ref":"Teiserver.Game.MatchTypeLib.html#get_match_type_by_name_or_id/1-examples"},{"type":"function","title":"Teiserver.Game.MatchTypeLib.get_or_create_match_type/1","doc":"Gets a match_type of a given name, if the match_type does not exist it is created.","ref":"Teiserver.Game.MatchTypeLib.html#get_or_create_match_type/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchTypeLib.get_or_create_match_type/1","doc":"iex> get_or_create_match_type(\"name\")\n    %MatchType{}\n\n    iex> get_or_create_match_type(\"not a match_type\"\")\n    %MatchType{}","ref":"Teiserver.Game.MatchTypeLib.html#get_or_create_match_type/1-examples"},{"type":"function","title":"Teiserver.Game.MatchTypeLib.list_match_types/1","doc":"Returns the list of match_types.","ref":"Teiserver.Game.MatchTypeLib.html#list_match_types/1"},{"type":"function","title":"Examples - Teiserver.Game.MatchTypeLib.list_match_types/1","doc":"iex> list_match_types()\n    [%MatchType{}, ...]","ref":"Teiserver.Game.MatchTypeLib.html#list_match_types/1-examples"},{"type":"function","title":"Teiserver.Game.MatchTypeLib.update_match_type/2","doc":"Updates a match_type.","ref":"Teiserver.Game.MatchTypeLib.html#update_match_type/2"},{"type":"function","title":"Examples - Teiserver.Game.MatchTypeLib.update_match_type/2","doc":"iex> update_match_type(match_type, %{field: new_value})\n    {:ok, %MatchType{}}\n\n    iex> update_match_type(match_type, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Game.MatchTypeLib.html#update_match_type/2-examples"},{"type":"module","title":"Teiserver.Settings.ServerSetting","doc":"# Site setting\nA key/value storage of settings used as part of the server.\n\n#","ref":"Teiserver.Settings.ServerSetting.html"},{"type":"module","title":"Attributes - Teiserver.Settings.ServerSetting","doc":"* `:key` - The key of the setting\n* `:email` - The value of the setting","ref":"Teiserver.Settings.ServerSetting.html#module-attributes"},{"type":"type","title":"Teiserver.Settings.ServerSetting.t/0","doc":"","ref":"Teiserver.Settings.ServerSetting.html#t:t/0"},{"type":"module","title":"Teiserver.Settings.ServerSettingLib","doc":"Library of server_setting related functions.","ref":"Teiserver.Settings.ServerSettingLib.html"},{"type":"function","title":"Teiserver.Settings.ServerSettingLib.change_server_setting/2","doc":"Returns an `%Ecto.Changeset{}` for tracking server_setting changes.","ref":"Teiserver.Settings.ServerSettingLib.html#change_server_setting/2"},{"type":"function","title":"Examples - Teiserver.Settings.ServerSettingLib.change_server_setting/2","doc":"iex> change_server_setting(server_setting)\n    %Ecto.Changeset{data: %ServerSetting{}}","ref":"Teiserver.Settings.ServerSettingLib.html#change_server_setting/2-examples"},{"type":"function","title":"Teiserver.Settings.ServerSettingLib.create_server_setting/1","doc":"Creates a server_setting.","ref":"Teiserver.Settings.ServerSettingLib.html#create_server_setting/1"},{"type":"function","title":"Examples - Teiserver.Settings.ServerSettingLib.create_server_setting/1","doc":"iex> create_server_setting(%{field: value})\n    {:ok, %ServerSetting{}}\n\n    iex> create_server_setting(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Settings.ServerSettingLib.html#create_server_setting/1-examples"},{"type":"function","title":"Teiserver.Settings.ServerSettingLib.delete_server_setting/1","doc":"Deletes a server_setting.","ref":"Teiserver.Settings.ServerSettingLib.html#delete_server_setting/1"},{"type":"function","title":"Examples - Teiserver.Settings.ServerSettingLib.delete_server_setting/1","doc":"iex> delete_server_setting(server_setting)\n    {:ok, %ServerSetting{}}\n\n    iex> delete_server_setting(server_setting)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Settings.ServerSettingLib.html#delete_server_setting/1-examples"},{"type":"function","title":"Teiserver.Settings.ServerSettingLib.get_server_setting/2","doc":"Gets a single server_setting.\n\nReturns nil if the ServerSetting does not exist.","ref":"Teiserver.Settings.ServerSettingLib.html#get_server_setting/2"},{"type":"function","title":"Examples - Teiserver.Settings.ServerSettingLib.get_server_setting/2","doc":"iex> get_server_setting(123)\n    %ServerSetting{}\n\n    iex> get_server_setting(456)\n    nil","ref":"Teiserver.Settings.ServerSettingLib.html#get_server_setting/2-examples"},{"type":"function","title":"Teiserver.Settings.ServerSettingLib.get_server_setting!/2","doc":"Gets a single server_setting.\n\nRaises `Ecto.NoResultsError` if the ServerSetting does not exist.","ref":"Teiserver.Settings.ServerSettingLib.html#get_server_setting!/2"},{"type":"function","title":"Examples - Teiserver.Settings.ServerSettingLib.get_server_setting!/2","doc":"iex> get_server_setting!(123)\n    %ServerSetting{}\n\n    iex> get_server_setting!(456)\n    ** (Ecto.NoResultsError)","ref":"Teiserver.Settings.ServerSettingLib.html#get_server_setting!/2-examples"},{"type":"function","title":"Teiserver.Settings.ServerSettingLib.list_server_settings/1","doc":"Returns the list of server_settings.","ref":"Teiserver.Settings.ServerSettingLib.html#list_server_settings/1"},{"type":"function","title":"Examples - Teiserver.Settings.ServerSettingLib.list_server_settings/1","doc":"iex> list_server_settings()\n    [%ServerSetting{}, ...]","ref":"Teiserver.Settings.ServerSettingLib.html#list_server_settings/1-examples"},{"type":"function","title":"Teiserver.Settings.ServerSettingLib.update_server_setting/2","doc":"Updates a server_setting.","ref":"Teiserver.Settings.ServerSettingLib.html#update_server_setting/2"},{"type":"function","title":"Examples - Teiserver.Settings.ServerSettingLib.update_server_setting/2","doc":"iex> update_server_setting(server_setting, %{field: new_value})\n    {:ok, %ServerSetting{}}\n\n    iex> update_server_setting(server_setting, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Settings.ServerSettingLib.html#update_server_setting/2-examples"},{"type":"module","title":"Teiserver.Settings.UserSetting","doc":"# User setting\nA key/value storage of settings tied to users\n\n#","ref":"Teiserver.Settings.UserSetting.html"},{"type":"module","title":"Attributes - Teiserver.Settings.UserSetting","doc":"* `:user_id` - A reference to the User in question\n* `:key` - The key of the setting\n* `:email` - The value of the setting","ref":"Teiserver.Settings.UserSetting.html#module-attributes"},{"type":"type","title":"Teiserver.Settings.UserSetting.t/0","doc":"","ref":"Teiserver.Settings.UserSetting.html#t:t/0"},{"type":"module","title":"Teiserver.Settings.UserSettingLib","doc":"Library of user_setting related functions.","ref":"Teiserver.Settings.UserSettingLib.html"},{"type":"function","title":"Teiserver.Settings.UserSettingLib.change_user_setting/2","doc":"Returns an `%Ecto.Changeset{}` for tracking user_setting changes.","ref":"Teiserver.Settings.UserSettingLib.html#change_user_setting/2"},{"type":"function","title":"Examples - Teiserver.Settings.UserSettingLib.change_user_setting/2","doc":"iex> change_user_setting(user_setting)\n    %Ecto.Changeset{data: %UserSetting{}}","ref":"Teiserver.Settings.UserSettingLib.html#change_user_setting/2-examples"},{"type":"function","title":"Teiserver.Settings.UserSettingLib.create_user_setting/1","doc":"Creates a user_setting.","ref":"Teiserver.Settings.UserSettingLib.html#create_user_setting/1"},{"type":"function","title":"Examples - Teiserver.Settings.UserSettingLib.create_user_setting/1","doc":"iex> create_user_setting(%{field: value})\n    {:ok, %UserSetting{}}\n\n    iex> create_user_setting(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Settings.UserSettingLib.html#create_user_setting/1-examples"},{"type":"function","title":"Teiserver.Settings.UserSettingLib.delete_user_setting/1","doc":"Deletes a user_setting.","ref":"Teiserver.Settings.UserSettingLib.html#delete_user_setting/1"},{"type":"function","title":"Examples - Teiserver.Settings.UserSettingLib.delete_user_setting/1","doc":"iex> delete_user_setting(user_setting)\n    {:ok, %UserSetting{}}\n\n    iex> delete_user_setting(user_setting)\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Settings.UserSettingLib.html#delete_user_setting/1-examples"},{"type":"function","title":"Teiserver.Settings.UserSettingLib.get_user_setting/2","doc":"Gets a single user_setting.\n\nReturns nil if the UserSetting does not exist.","ref":"Teiserver.Settings.UserSettingLib.html#get_user_setting/2"},{"type":"function","title":"Examples - Teiserver.Settings.UserSettingLib.get_user_setting/2","doc":"iex> get_user_setting(123)\n    %UserSetting{}\n\n    iex> get_user_setting(456)\n    nil","ref":"Teiserver.Settings.UserSettingLib.html#get_user_setting/2-examples"},{"type":"function","title":"Teiserver.Settings.UserSettingLib.get_user_setting!/2","doc":"Gets a single user_setting.\n\nRaises `Ecto.NoResultsError` if the UserSetting does not exist.","ref":"Teiserver.Settings.UserSettingLib.html#get_user_setting!/2"},{"type":"function","title":"Examples - Teiserver.Settings.UserSettingLib.get_user_setting!/2","doc":"iex> get_user_setting!(123)\n    %UserSetting{}\n\n    iex> get_user_setting!(456)\n    ** (Ecto.NoResultsError)","ref":"Teiserver.Settings.UserSettingLib.html#get_user_setting!/2-examples"},{"type":"function","title":"Teiserver.Settings.UserSettingLib.list_user_settings/1","doc":"Returns the list of user_settings.","ref":"Teiserver.Settings.UserSettingLib.html#list_user_settings/1"},{"type":"function","title":"Examples - Teiserver.Settings.UserSettingLib.list_user_settings/1","doc":"iex> list_user_settings()\n    [%UserSetting{}, ...]","ref":"Teiserver.Settings.UserSettingLib.html#list_user_settings/1-examples"},{"type":"function","title":"Teiserver.Settings.UserSettingLib.update_user_setting/2","doc":"Updates a user_setting.","ref":"Teiserver.Settings.UserSettingLib.html#update_user_setting/2"},{"type":"function","title":"Examples - Teiserver.Settings.UserSettingLib.update_user_setting/2","doc":"iex> update_user_setting(user_setting, %{field: new_value})\n    {:ok, %UserSetting{}}\n\n    iex> update_user_setting(user_setting, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Teiserver.Settings.UserSettingLib.html#update_user_setting/2-examples"},{"type":"module","title":"Teiserver.Helpers.MapHelper","doc":"Helper functions for maps","ref":"Teiserver.Helpers.MapHelper.html"},{"type":"function","title":"Teiserver.Helpers.MapHelper.map_diffs/2","doc":"Returns a list of changes from m1 to m2. Does not handle deleted keys.","ref":"Teiserver.Helpers.MapHelper.html#map_diffs/2"},{"type":"behaviour","title":"Teiserver.Migration","doc":"Migrations create and modify the database tables Teiserver needs to function.","ref":"Teiserver.Migration.html"},{"type":"behaviour","title":"Usage - Teiserver.Migration","doc":"To use migrations in your application you'll need to generate an `Ecto.Migration` that wraps\ncalls to `Teiserver.Migration`:\n\n```bash\nmix ecto.gen.migration add_teiserver\n```\n\nOpen the generated migration in your editor and call the `up` and `down` functions on\n`Teiserver.Migration`:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddTeiserver do\n  use Ecto.Migration\n\n  def up, do: Teiserver.Migrations.up()\n\n  def down, do: Teiserver.Migrations.down()\nend\n```\n\nThis will run all of Teiserver's versioned migrations for your database.\n\nNow, run the migration to create the table:\n\n```bash\nmix ecto.migrate\n```\n\nMigrations between versions are idempotent. As new versions are released, you may need to run\nadditional migrations. To do this, generate a new migration:\n\n```bash\nmix ecto.gen.migration upgrade_Teiserver_to_v1\n```\n\nOpen the generated migration in your editor and call the `up` and `down` functions on\n`Teiserver.Migration`, passing a version number:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.UpgradeTeiserverToV1 do\n  use Ecto.Migration\n\n  def up, do: Teiserver.Migrations.up(version: 1)\n\n  def down, do: Teiserver.Migrations.down(version: 1)\nend\n```","ref":"Teiserver.Migration.html#module-usage"},{"type":"behaviour","title":"Isolation with Prefixes - Teiserver.Migration","doc":"Teiserver supports namespacing through PostgreSQL schemas, also called \"prefixes\" in Ecto. With\nprefixes your jobs table can reside outside of your primary schema (usually public) and you can\nhave multiple separate job tables.\n\nTo use a prefix you first have to specify it within your migration:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPrefixedTeiserverJobsTable do\n  use Ecto.Migration\n\n  def up, do: Teiserver.Migrations.up(prefix: \"private\")\n\n  def down, do: Teiserver.Migrations.down(prefix: \"private\")\nend\n```\n\nThe migration will create the \"private\" schema and all tables, functions and triggers within\nthat schema. With the database migrated you'll then specify the prefix in your configuration:\n\n```elixir\nconfig :my_app, Teiserver,\n  prefix: \"private\",\n  ...\n```\n\nIn some cases, for example if your \"private\" schema already exists and your database user in\nproduction doesn't have permissions to create a new schema, trying to create the schema from the\nmigration will result in an error. In such situations, it may be useful to inhibit the creation\nof the \"private\" schema:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPrefixedTeiserverJobsTable do\n  use Ecto.Migration\n\n  def up, do: Teiserver.Migrations.up(prefix: \"private\", create_schema: false)\n\n  def down, do: Teiserver.Migrations.down(prefix: \"private\")\nend\n```","ref":"Teiserver.Migration.html#module-isolation-with-prefixes"},{"type":"behaviour","title":"Migrating Without Ecto - Teiserver.Migration","doc":"If your application uses something other than Ecto for migrations, be it an external system or\nanother ORM, it may be helpful to create plain SQL migrations for Teiserver database schema changes.\n\nThe simplest mechanism for obtaining the SQL changes is to create the migration locally and run\n`mix ecto.migrate --log-migrations-sql`. That will log all of the generated SQL, which you can\nthen paste into your migration system of choice.","ref":"Teiserver.Migration.html#module-migrating-without-ecto"},{"type":"function","title":"Teiserver.Migration.down/1","doc":"Run the `down` changes for all migrations between the current version and the initial version.","ref":"Teiserver.Migration.html#down/1"},{"type":"function","title":"Example - Teiserver.Migration.down/1","doc":"Run all migrations from current version down to the first:\n\n    Teiserver.Migration.down()\n\nRun migrations down to and including a specified version:\n\n    Teiserver.Migration.down(version: 5)\n\nRun migrations in an alternate prefix:\n\n    Teiserver.Migration.down(prefix: \"game_server\")","ref":"Teiserver.Migration.html#down/1-example"},{"type":"callback","title":"Teiserver.Migration.down/1","doc":"Migrates storage down to the previous version.","ref":"Teiserver.Migration.html#c:down/1"},{"type":"function","title":"Teiserver.Migration.migrated_version/1","doc":"Check the latest version the database is migrated to.","ref":"Teiserver.Migration.html#migrated_version/1"},{"type":"function","title":"Example - Teiserver.Migration.migrated_version/1","doc":"Teiserver.Migration.migrated_version()","ref":"Teiserver.Migration.html#migrated_version/1-example"},{"type":"callback","title":"Teiserver.Migration.migrated_version/1","doc":"Identifies the last migrated version.","ref":"Teiserver.Migration.html#c:migrated_version/1"},{"type":"function","title":"Teiserver.Migration.up/1","doc":"Run the `up` changes for all migrations between the initial version and the current version.","ref":"Teiserver.Migration.html#up/1"},{"type":"function","title":"Example - Teiserver.Migration.up/1","doc":"Run all migrations up to the current version:\n\n    Teiserver.Migration.up()\n\nRun migrations up to a specified version:\n\n    Teiserver.Migration.up(version: 2)\n\nRun migrations in an alternate prefix:\n\n    Teiserver.Migration.up(prefix: \"game_server\")\n\nRun migrations in an alternate prefix but don't try to create the schema:\n\n    Teiserver.Migration.up(prefix: \"game_server\", create_schema: false)","ref":"Teiserver.Migration.html#up/1-example"},{"type":"callback","title":"Teiserver.Migration.up/1","doc":"Migrates storage up to the latest version.","ref":"Teiserver.Migration.html#c:up/1"},{"type":"module","title":"TeiserverMacros","doc":"A set of macros for defining common file types.\n\nThis can be used in your application as:\n\n    use TeiserverMacros, :queries\n    use TeiserverMacros, :library","ref":"TeiserverMacros.html"},{"type":"macro","title":"TeiserverMacros.__using__/1","doc":"When used, dispatch to the appropriate controller/view/etc.","ref":"TeiserverMacros.html#__using__/1"},{"type":"function","title":"TeiserverMacros.library/0","doc":"","ref":"TeiserverMacros.html#library/0"},{"type":"function","title":"TeiserverMacros.queries/0","doc":"","ref":"TeiserverMacros.html#queries/0"},{"type":"function","title":"TeiserverMacros.schema/0","doc":"","ref":"TeiserverMacros.html#schema/0"},{"type":"extras","title":"Changelog","doc":"","ref":"changelog.html"},{"type":"extras","title":"v0.0.4 - Changelog","doc":"- Added `Lobby`, `LobbySummary`, `MatchType`, `Match`, `MatchMembership`, `MatchSettingType`, `MatchSetting` schemas, libs and queries\n- Added pubsub events for clients connecting, disconnecting and process destruction\n- Added lobby pubsub events\n- Split client updates into three types (out of lobby, in lobby and full)","ref":"changelog.html#v0-0-4"},{"type":"extras","title":"v0.0.3 - Changelog","doc":"- Added `Teiserver.Api` to make getting writing a basic protocol handler easier\n- Added `Room`, `RoomMessage` and `DirectMessage` along with their relevant lib functions\n- Added `UserLib.allow?/2` and `UserLib.restricted?/2`\n- Added `Teiserver.Communications` and relevant functions for Rooms, RoomMessages and DirectMessages\n- Added tests for query libraries, caught some bugs in the process","ref":"changelog.html#v0-0-3"},{"type":"extras","title":"v0.0.2 - Changelog","doc":"- Client connectivity functionality\n- Expanded docs significantly\n- Filled in roadmap\n- Stubs for most context files\n- Added unit tests","ref":"changelog.html#v0-0-2"},{"type":"extras","title":"v0.0.1 - Changelog","doc":"- Proof of concept user functionality\n- Created basic hello world example","ref":"changelog.html#v0-0-1"},{"type":"extras","title":"Installation","doc":"# Installation\nA middleware server library.","ref":"installation.html"},{"type":"extras","title":"Dependency - Installation","doc":"First add to your dependencies in `mix.exs`.\n```elixir\ndef deps do\n  [\n    {:teiserver, \"~> 0.0.4\"}\n  ]\nend\n```","ref":"installation.html#dependency"},{"type":"extras","title":"Migration - Installation","doc":"Now add a migration\n```bash\nmix ecto.gen.migration add_teiserver_tables\n```\n\nOpen the generated migration and add the below code:\n```elixir\ndefmodule MyApp.Repo.Migrations.AddTeiserverTables do\n  use Ecto.Migration\n\n  def up do\n    Teiserver.Migration.up()\n  end\n\n  # We specify `version: 1` in `down`, ensuring that we'll roll all the way back down if\n  # necessary, regardless of which version we've migrated `up` to.\n  def down do\n    Teiserver.Migration.down(version: 1)\n  end\nend\n```","ref":"installation.html#migration"},{"type":"extras","title":"Supervision tree - Installation","doc":"Add this to your Application supervision tree:\n```elixir\nchildren = [\n  {Teiserver, Application.get_env(:my_app, Teiserver)}\n]\n```","ref":"installation.html#supervision-tree"},{"type":"extras","title":"Config - Installation","doc":"In `config.exs`\n```elixir\nconfig :teiserver,\n  repo: MyApp.Repo\n```","ref":"installation.html#config"},{"type":"extras","title":"Config options","doc":"# Config options\nTeiserver config options all exist in the `:teiserver` namespace such as:\n\n```elixir\nconfig :teiserver,\n  repo: HelloWorldServer.Repo,\n  client_destroy_timeout_seconds: 300\n```\n\n## `repo` - Required\nThe repo used by Teiserver.\n\n## `client_destroy_timeout_seconds` - Default: `300`\nAfter all connections disconnect a client server process will terminate itself. This value is the number of seconds after that last disconnection and the termination. If a connection is re-established the timeout is cancelled and restarted from 0 if all connections later disconnect again.\n\n## `lobby_join_method` - Default: `:simple`\nThe process for which players are able to join lobbies, can be set to `:simple` or `:host_approval`.\n\n`:simple` means all requests to join lobbies are approved by Teiserver.\n\n`:host_approval` means all requests to join lobbies are first approved by Teiserver but then approved by the host client. This means you will need to implement the relevant messaging and handling for clients hosting lobbies to be able to approve join requests.\n\n# Users","ref":"config.html"},{"type":"extras","title":"default_behaviour_score - Default: `10_000` - Config options","doc":"The `behaviour_score` given to users registered using `Teiserver.Account.UserLib.register_user/1`.","ref":"config.html#default_behaviour_score-default-10_000"},{"type":"extras","title":"default_trust_score - Default: `10_000` - Config options","doc":"The `trust_score` given to users registered using `Teiserver.Account.UserLib.register_user/1`.","ref":"config.html#default_trust_score-default-10_000"},{"type":"extras","title":"default_social_score - Default: `10_000` - Config options","doc":"The `social_score` given to users registered using `Teiserver.Account.UserLib.register_user/1`.","ref":"config.html#default_social_score-default-10_000"},{"type":"extras","title":"default_min_user_password_length - Default: 6 - Config options","doc":"The minimum length for a user password.\n\n# Clustering\n## `teiserver_clustering` - Default: true\nWhen enabled Teiserver will attempt to handle the clustering of nodes using a database table. Turning it off will mean this behaves like any other application and you can either not cluster it or use things like `libcluster` as you desire. See `Teiserver.System.ClusterManager` for more details.\n\n## `teiserver_clustering_post_join_functions` - Default: []\nWhen teiserver_clustering is enabled, this will be a list of functions called by the genserver handling the join once it has joined the cluster. See `Teiserver.System.ClusterManager` for more details.\n\n# Function overrides\nTeiserver implements some defaults you may want to overwrite.\n\n## `fn_calculate_match_type`\nAllows you to overwrite `Teiserver.Game.MatchTypeLib.default_calculate_match_type/1`. This is used to determine the type assigned to each match. By default a 2 player game is a `Duel` and anything else is a `Team` game.\n\n## `fn_calculate_user_permissions`\nAllows you to overwrite `Teiserver.Account.User.default_calculate_user_permissions/1`. This is used to generate the list of permissions held by a user. By default it mirrors their groups.\n\n## `fn_lobby_name_acceptor`\nA function used to determine if a lobby name is acceptable. Defaults to `Teiserver.Game.LobbyLib.default_lobby_name_acceptable/1` which always returns true.\n\n## `fn_user_name_acceptor`\nA function used to determine if a lobby name is acceptable. Defaults to `Teiserver.Account.UserLib.default_user_name_acceptable/1` which always returns true.\n\n## `fn_uuid_generator`\nThe function used to generate UUIDs. Defaults to `&Ecto.UUID.generate/0`.\n\n\n# Complete example config\n```elixir\nconfig :teiserver,\n  repo: HelloWorldServer.Repo,\n  client_destroy_timeout_seconds: 300,\n  lobby_join_method: :simple,\n  teiserver_clustering: true,\n\n  # Users\n  default_behaviour_score: 10_000,\n  default_trust_score: 10_000,\n  default_social_score: 10_000,\n  default_min_user_password_length: 6,\n\n  # Overrides\n  fn_calculate_match_type: &HelloWorldServer.Game.calculate_match_type/1,\n  fn_calculate_user_permissions: &HelloWorldServer.Account.calculate_user_permissions/1,\n\n  fn_lobby_name_acceptor: &HelloWorldServer.Account.lobby_name_acceptor/1,\n  fn_user_name_acceptor: &HelloWorldServer.Account.user_name_acceptor/1,\n\n  fn_uuid_generator: &Ecto.UUID.generate/0\n```","ref":"config.html#default_min_user_password_length-default-6"},{"type":"extras","title":"Hello World","doc":"# Hello World\nThe purpose of this guide is to show you how to create a very basic server implementing the commands `ping`, `register`, `login`, `whois`, `whoami`, `users` and `clients`. We will be using [ranch](https://ninenines.eu/docs/en/ranch/1.8/manual/) to provide a plain-text interface to our server.\n\nRanch is a tcp server and will allow us to provide an easy to test and prototype interface. For a more serious project you'll probably want to be looking at more structured data; for this example plain-text is fine.","ref":"hello_world.html"},{"type":"extras","title":"Create project - Hello World","doc":"```bash\nmix new --sup hello_world_server\n```","ref":"hello_world.html#create-project"},{"type":"extras","title":"Add dependencies - Hello World","doc":"In `mix.exs`\n```elixir\ndef deps do\n  [\n    {:teiserver, path: \"../teiserver\"},\n    {:ecto_sql, \"~> 3.10\"},\n    {:postgrex, \">= 0.0.0\"},\n    {:thousand_island, \"~> 1.3\"}\n  ]\nend\n```\n\nNow get the relevant dependencies\n\n```bash\nmix deps.get && mix compile\n```","ref":"hello_world.html#add-dependencies"},{"type":"extras","title":"Create a migration - Hello World","doc":"```bash\nmix ecto.gen.migration add_teiserver_tables\n```\n\nAnd populate it like so:\n```elixir\ndefmodule HelloWorldServer.Repo.Migrations.AddTeiserverTables do\n  use Ecto.Migration\n\n  def up do\n    Teiserver.Migration.up()\n  end\n\n  def down do\n    Teiserver.Migration.down(version: 1)\n  end\nend\n```","ref":"hello_world.html#create-a-migration"},{"type":"extras","title":"Application files - Hello World","doc":"`lib/hello_world_server/repo.ex`\n```elixir\ndefmodule HelloWorldServer.Repo do\n  use Ecto.Repo,\n    otp_app: :hello_world_server,\n    adapter: Ecto.Adapters.Postgres\nend\n```\n\n`config/config.exs`\n```elixir\nimport Config\n\nconfig :hello_world_server,\n  ecto_repos: [HelloWorldServer.Repo]\n\nconfig :hello_world_server, HelloWorldServer.Repo,\n  database: \"hello_world_server\",\n  username: \"postgres\",\n  password: \"postgres\",\n  hostname: \"localhost\"\n\nconfig :teiserver,\n  repo: HelloWorldServer.Repo\n```\n\nEdit `application.ex` to start the various components on startup.\n```elixir\n  children = [\n    HelloWorldServer.Repo,\n    {Ecto.Migrator,\n        repos: Application.fetch_env!(:hello_world_server, :ecto_repos),\n        skip: System.get_env(\"SKIP_MIGRATIONS\") == \"true\"},\n\n    {ThousandIsland, port: 8200, handler_module: HelloWorldServer.TcpServer}\n  ]\n```","ref":"hello_world.html#application-files"},{"type":"extras","title":"Add our tcp server - Hello World","doc":"Place in `lib/hello_world_server/tcp_server.ex`\n```elixir\ndefmodule HelloWorldServer.TcpServer do\n  use ThousandIsland.Handler\n  alias HelloWorldServer.{TcpIn, TcpOut}\n\n  @impl ThousandIsland.Handler\n  def handle_connection(socket, _state) do\n    {:continue, %{\n      user_id: nil,\n      socket: socket\n    }}\n  end\n\n  @impl ThousandIsland.Handler\n  # If Ctrl + C is sent through it kills the connection, makes telnet debugging easier\n  def handle_data(<<255, 244, 255, 253, 6>>, _socket, state) do\n    {:close, state}\n  end\n\n  def handle_data(data, socket, state) do\n    {new_state, response} = TcpIn.data_in(String.trim(data), state)\n    TcpOut.data_out(response, new_state)\n    {:continue, new_state}\n  end\nend\n```","ref":"hello_world.html#add-our-tcp-server"},{"type":"extras","title":"Data in - Hello World","doc":"Place in `lib/hellow_world_server/tcp_in.ex`, this will handle all the commands coming in.\n```elixir\ndefmodule HelloWorldServer.TcpIn do\n  alias Teiserver.Api\n\n  def data_in(\"ping\" <> _data, state) do\n    {state, \"pong\"}\n  end\n\n  def data_in(\"login \" <> data, state) do\n    [name, password] = String.split(data, \" \")\n    case Api.maybe_authenticate_user(name, password) do\n      {:ok, user} ->\n        Api.connect_user(user)\n        {%{state | user_id: user.id}, \"You are now logged in as '#{user.name}'\"}\n      {:error, :no_user} ->\n        {state, \"Login failed (no user)\"}\n      {:error, :bad_password} ->\n        {state, \"Login failed (bad password)\"}\n    end\n  end\n\n  def data_in(\"register \" <> data, state) do\n    [name, password] = String.split(data, \" \")\n    \n    # We're not using emails right now but Teiserver expects them to be unique\n    # this will do for the purposes of this example\n    email = to_string(:rand.uniform())\n    \n    case Api.register_user(name, email, password) do\n      {:ok, _user} ->\n        {state, \"User created, you can now login with 'login name password'\"}\n      {:error, _} ->\n        {state, \"Error registering user\"}\n    end\n  end\n\n  def data_in(\"whois \" <> data, state) do\n    case Teiserver.Account.get_user_by_name(data) do\n      nil ->\n        {state, \"I cannot find a user by the name of '#{data}'\"}\n      user ->\n        {state, \"User #{user.name} exists with an ID of #{user.id}\"}\n    end\n  end\n\n  def data_in(\"whoami\" <> _data, %{user_id: user_id} = state) do\n    case Teiserver.Account.get_user_by_id(user_id) do\n      nil ->\n        {state, \"You are not logged in\"}\n      user ->\n        {state, \"You are '#{user.name}'\"}\n    end\n  end\n\n  def data_in(\"users\" <> _data, state) do\n    names = Teiserver.Account.list_users(select: [:name])\n    |> Enum.map(fn %{name: name} -> name end)\n    |> Enum.join(\", \")\n\n    {state, \"User names: #{names}\"}\n  end\n\n  def data_in(\"clients\" <> _data, state) do\n    client_ids = Teiserver.Connections.list_client_ids()\n\n    names = Teiserver.Account.list_users(where: [id_in: client_ids], select: [:name])\n    |> Enum.map(fn %{name: name} -> name end)\n    |> Enum.join(\", \")\n\n    {state, \"Client names: #{names}\"}\n  end\nend\n```","ref":"hello_world.html#data-in"},{"type":"extras","title":"Data out - Hello World","doc":"Place in `lib/hellow_world_server/tcp_out.ex`, this will handle sending data back to our users.\n```elixir\ndefmodule HelloWorldServer.TcpOut do\n  def data_out(msg, state) do\n    ThousandIsland.Socket.send(state.socket, \"#{msg}\\n\")\n  end\nend\n```\n\n# Showtime!\nGet deps\nWe now need to run our application.\n```bash\niex -S mix run --no-halt\n```\n\nThis will open up a REPL into the application and we can run queries manually if we want to such as:\n```elixir\nTeisever.Account.list_users()\nTeisever.Account.list_clients()\n```\n\n#","ref":"hello_world.html#data-out"},{"type":"extras","title":"Telnet - Hello World","doc":"In another terminal we can then do:\n```bash\ntelnet localhost 8200\n# Trying 127.0.0.1...\n# Connected to localhost.\n# Escape character is '^]'.\n\nping\n# pong\n\nwhoami\n# You are not logged in\n\nwhois teifion\n# I cannot find a user by the name of 'Teifion'\n\nregister teifion password1\n# User created, you can now login with 'login name password'\n\nwhois teifion\n# User Teifion exists with an ID of 1\n\nlogin teifion nopass\n# Login failed (bad password)\n\nlogin teifion password1\n# You are now logged in as Teifion\n\nwhoami\n# You are 'Teifion'\n\nregister bob password1\n# User created, you can now login with 'login name password'\n\nusers\n# User names: teifion, bob\n\nclients\n# Client names: teifion\n```","ref":"hello_world.html#telnet"},{"type":"extras","title":"Program structure","doc":"# Program structure","ref":"program_structure.html"},{"type":"extras","title":"User to Teiserver - Program structure","doc":"The expectation is your users will have an application running on their machine, this application will connect to your endpoint (websocket, grpc, plaintext etc). Your endpoint will be an Elixir application running Teiserver as a dependency and making the calls to it.\n\n```mermaid\ngraph TD;\n    app[User application] <--> Endpoint;\n    Endpoint --> CommandIn[Command handler];\n    CommandIn --> Teiserver[Teiserver library];\n    Teiserver --> HandleOut[Response handler];\n    HandleOut --> Endpoint;\n```","ref":"program_structure.html#user-to-teiserver"},{"type":"extras","title":"Endpoint information flow - Program structure","doc":"The endpoints are can communicate with users however you want. In terms of Teiserver they are expected to make function calls to Teiserver context modules and receive information via return values and also process messages (typically via Pubsubs).\n\n```mermaid\nsequenceDiagram\n    actor User\n    participant Endpoint\n    participant Teiserver\n    \n    Note right of User: User event\n    User->>Endpoint: Message/Command\n    Endpoint->>Teiserver: Function call\n    Teiserver->>Endpoint: Return value\n    Endpoint->>User: Message/Command\n    \n    Note right of User: Server event\n    Teiserver->>Endpoint: Pubsub message\n    Endpoint->>User: Message/Command\n```","ref":"program_structure.html#endpoint-information-flow"},{"type":"extras","title":"Login through to playing a game - Program structure","doc":"The expected flow of events for a user logging into your application and playing a game is expected to look something like this. Note depending on protocols and features you may end up with slightly different steps.\n\n```mermaid\nsequenceDiagram\n    actor User\n    participant Server\n    participant Game\n    \n    Game->>+Server: Login\n    Server->>Game: Accept login\n    Game->>Server: Open lobby\n    \n    User->>Server: Login\n    Server->>User: Accept login\n    Server->>User: Send client info\n    \n    User->>Server: Request lobby list\n    Server->>User: Send lobby list\n    \n    User->>Server: Join lobby\n    Server->>Game: User joined\n    Game->>Server: Start match\n    Server->>-User: Start match\n    \n    Note right of Server: Server not needed now\n    \n    User->>Game: Match events\n    Game->>User: Match events\n    User->>Game: Match events\n    Game->>User: Match events\n    \n    Game->>+Server: Match ended\n    \n    Note right of Server: Server becomes relevant\n    \n    User->>Server: Update client state\n    User->>Server: Leave lobby\n    Server->>-Game: User left\n```","ref":"program_structure.html#login-through-to-playing-a-game"},{"type":"extras","title":"Snippets","doc":"# Snippets","ref":"snippets.html"},{"type":"extras","title":"User authentication and login - Snippets","doc":"Given a `name` and `password` we want to log a user in.\n\n```elixir\n@spec connect_user(String.t(), String.t()) :: {:ok, Teiserver.Account.User.t()} | {:error, String.t()}\ndef connect_user(name, password) do\n  case Teiserver.Account.get_user_by_name(name) do\n    nil ->\n      {:error, \"User not found\"}\n    user ->\n      if Teiserver.Account.valid_password?(user, password) do\n        Teiserver.Connections.connect_user(user)\n        {:ok, user}\n      else\n        {:error, \"Invalid password\"}\n      end\n  end\nend\n```","ref":"snippets.html#user-authentication-and-login"},{"type":"extras","title":"Match lifecycle","doc":"# Match lifecycle\nThe main purpose of Teiserver is to facilitate running a game; as a result the Lobby and Match systems are one of the core features and have a lot of customisation and flexibility.\n\nIn every case where the `Game` context is used there is a similar or identical function in `Api`.\n\n#","ref":"match_lifecycle.html"},{"type":"extras","title":"Overview - Match lifecycle","doc":"There are multiple stages to a match taking place, they will typically follow the below diagram.\n```mermaid\ngraph LR;\n    create[Lobby opened] --> join[Users join];\n    join --> configure[Lobby configured];\n    configure --> begin[Match starts];\n    begin --> play[Game is played];\n    play --> finish[Match ends];\n    finish --> postgame[Post game];\n```","ref":"match_lifecycle.html#overview"},{"type":"extras","title":"Open lobby - Match lifecycle","doc":"A call to either `Teiserver.Game.open_lobby/2` is used to open the lobby. You must pass in the user_id of the host (who must also be connected) and the initial name for the lobby.\n\nThis will create a `Teiserver.Game.LobbyServer` process to store the state of the lobby. When the lobby is opened a minimal `Teiserver.Game.Match` object is created and the lobby process updated to point to it.","ref":"match_lifecycle.html#open-lobby"},{"type":"extras","title":"Users joining - Match lifecycle","doc":"After creating a lobby; users will be able to join. The nature of which will depend on your config [`lobby_join_method`](config.html#lobby_join_method-default-simple).\n\n#","ref":"match_lifecycle.html#users-joining"},{"type":"extras","title":"Joining - Match lifecycle","doc":"To add a client to the lobby you will need to call `Teiserver.Game.add_client_to_lobby/2`. Assuming this is successful the following will take place:\n- The lobby state will be updated to add this user to the member list and the spectator list\n- The lobby process will subscribe to updates about the client\n- The client process will be updated with default lobby details\n- Subscribers to the lobby pubsub will receive information about this including a `shared_secret`. This secret is passed on to the user themselves and the host to allow for authentication outside of the server connection.\n\n#","ref":"match_lifecycle.html#joining"},{"type":"extras","title":"Leaving - Match lifecycle","doc":"To remove a client from a lobby you will need to call `Teiserver.Game.remove_client_from_lobby/2`. When this is called the following takes place:\n- The client process is updated to show they are no longer in the lobby\n- The lobby state will be updated to remove this user from the member, spectator and player lists as appropriate\n\n#","ref":"match_lifecycle.html#leaving"},{"type":"extras","title":"Client updates - Match lifecycle","doc":"Typically a client will update via `Teiserver.Connections.update_client/2` but if you want to update the lobby details of a client you should use `Teiserver.Connections.update_client_in_lobby/2`.\n\nThe standard `update_client` only contacts the ClientServer to update values but with the `update_client_in_lobby` function the ClientServer will check with the LobbyServer before updating any details to ensure it is allowed to.","ref":"match_lifecycle.html#client-updates"},{"type":"extras","title":"Lobby configuration - Match lifecycle","doc":"> ##","ref":"match_lifecycle.html#lobby-configuration"},{"type":"extras","title":"Not fully implemented {: .info} - Match lifecycle","doc":">\n> Options for configuration through votes will be added and will result in additional details in this section.\n\nLobbies have a key `:game_settings` which holds key-value map of the settings chosen for the upcoming match. These can be changed at any time prior to the match starting.","ref":"match_lifecycle.html#not-fully-implemented-info"},{"type":"extras","title":"Match start - Match lifecycle","doc":"When the match is started and the users move into playing the game itself (which does not take place on the middleware server) a call needs to be made to `Api` or `Game` `start_match/1` (delegated to `Teiserver.Game.MatchLib.start_match/1`).\n\nThis will update the previously added Match object with the relevant information from the Lobby. It will create `Teiserver.Game.MatchMembership` objects for each player, create the relevant `Teiserver.Game.MatchSetting` objects (along with types) and update the Lobby to show the match as ongoing.","ref":"match_lifecycle.html#match-start"},{"type":"extras","title":"Playing the game - Match lifecycle","doc":"While the game is ongoing the server is not directly involved except for anything the host wishes to relay to the server such as public chat or game telemetry events.","ref":"match_lifecycle.html#playing-the-game"},{"type":"extras","title":"Match end - Match lifecycle","doc":"At the conclusion of the match players should be returned to the lobby interface and the `Api` or `Game` `end_match/2` (delegated to `Teiserver.Game.MatchLib.end_match/2`) should be called.\n\nThis will update both the Match object and MatchMembership objects with the relevant data.","ref":"match_lifecycle.html#match-end"},{"type":"extras","title":"Post game - Match lifecycle","doc":"#","ref":"match_lifecycle.html#post-game"},{"type":"extras","title":"Close - Match lifecycle","doc":"In some cases you will want the lobby to close in which case `Api` or `Game` `close_lobby/1` (delegated to `Teiserver.Game.LobbyLib.close_lobby/1`) should be called. This will remove everybody from the lobby and stop the lobby process.\n\n#","ref":"match_lifecycle.html#close"},{"type":"extras","title":"Cycle - Match lifecycle","doc":"If you wish to keep the lobby in existence you should call `Api` or `Game` `cycle_lobby/1` (delegated to `Teiserver.Game.LobbyLib.cycle_lobby/1`) which will create a new empty Match object for the new upcoming match.","ref":"match_lifecycle.html#cycle"},{"type":"extras","title":"Feature list","doc":"# Feature list","ref":"features.html"},{"type":"extras","title":"Accounts - Feature list","doc":"#","ref":"features.html#accounts"},{"type":"extras","title":"Users - Feature list","doc":"- Create, Read, Update, Delete\n- Password verification/storage\n- Free-form extra data fields","ref":"features.html#users"},{"type":"extras","title":"Connections - Feature list","doc":"#","ref":"features.html#connections"},{"type":"extras","title":"Clients - Feature list","doc":"- Tracking","ref":"features.html#clients"},{"type":"extras","title":"Communication - Feature list","doc":"- Room messages\n- Direct (user to user) messages\n- Lobby/Match messages","ref":"features.html#communication"},{"type":"extras","title":"Games - Feature list","doc":"#","ref":"features.html#games"},{"type":"extras","title":"Lobbies - Feature list","doc":"- Game agnostic\n\n#","ref":"features.html#lobbies"},{"type":"extras","title":"Matches - Feature list","doc":"- Game agnostic\n- Player tracking\n- Per-match settings","ref":"features.html#matches"},{"type":"extras","title":"Settings - Feature list","doc":"- Site settings\n- User settings","ref":"features.html#settings"},{"type":"extras","title":"Roadmap","doc":"# Roadmap\nThis is not a rigid document, it is (especially at this stage) liable to change. Please feel free to raise an issue if you have requests related to this.","ref":"roadmap.html"},{"type":"extras","title":"v0.1 plan - Roadmap","doc":"* `v0.0.1` - Basic user features (Account)\n* `v0.0.2` - Client features (Connections)\n* `v0.0.3` - Chat and Messaging (Communication)\n* `v0.0.4` - Lobbies (Lobby and Game)\n* `v0.0.5` - Logging and Event telemetry (Telemetry and Logging)\n* `v0.0.6` - Player relationships (Account and Community)\n* `v0.0.7` - Moderation (Moderation and Account)\n* `v0.0.8` - Parties (Community)\n* `v0.0.9` - Site and User Settings (Settings)\n* `v0.1` - Stability, tests and better examples on how to use it\n\nAt v0.1 I want the server to be in a state where developers can start to make use of it.","ref":"roadmap.html#v0-1-plan"},{"type":"extras","title":"v1.0 planned features - Roadmap","doc":"- Accounts\n  - Users\n  - User settings\n\n- Communication\n  - Chat rooms\n  - Direct messages\n\n- Community\n  - Player relationships (friend and avoid)\n\n- Connections\n  - Clients (connect, disconnect, update)\n\n- Lobby\n  - Opening/Closing\n  - Joining/Leaving\n\n- Moderation\n  - Ability to apply restrictions\n\n- Settings\n  - Site settings\n  - User settings\n\n- Telemetry\n  - In game events\n  - Server events\n  - Lobby events\n  - User events","ref":"roadmap.html#v1-0-planned-features"},{"type":"extras","title":"Planned/intended features - Roadmap","doc":"- Administration\n  - Moderation\n  - Analytics and Reporting\n- Accounts\n  - Users\n  - User settings\n  - Achievements\n  - Accolades (player awarded badges)\n- Communication\n  - Chat rooms\n  - Direct messages\n- Community\n  - Player relationships (e.g. friends)\n  - Player feedback system\n  - News/Miniblog\n  - Clans\n- Game related\n  - Lobbies\n  - Match history\n  - Leaderboards\n  - Matchmaking\n- Logging\n  - Daily numbers\n  - Aggregated numbers for weeks, months, quarters and years\n  - Crash and bug reports/player facing issue tracker\n- Telemetry\n  - In game events\n  - Server events\n  - Lobby events\n  - User events\n- Integrations\n  - [Steam](https://partner.steamgames.com/doc/webapi_overview)\n  - [Itch.io](https://itch.io/docs/api/serverside)\n  - [Discord](https://hex.pm/packages/nostrum)\n  - [Epic](https://dev.epicgames.com/docs/web-api-ref)","ref":"roadmap.html#planned-intended-features"},{"type":"extras","title":"Other - Roadmap","doc":"These features are intended/hoped for but will likely be in their own library:\n- Admin web interface\n- 3rd party API (Rest, GraphQL etc)","ref":"roadmap.html#other"},{"type":"extras","title":"Client messages","doc":"# Client messages\nPubSub messages sent out in relation to changes to clients.\n\n## `Teiserver.Connections.Client:{user_id}`\nThe pubsub messages received by anybody watching this account.\n\n#","ref":"client.html"},{"type":"extras","title":"Updated client - `:client_updated` - Client messages","doc":"Sent whenever the client in question is updated.\n\n- `:reason` - The reason for the change\n- `:user_id` - The ID of the user (which should also be present in the topic)\n- `:changes` - A map of the changed values\n\n```elixir\n%{\n  event: :client_updated,\n  user_id: User.id(),\n  changes: map()\n}\n```\n\n#","ref":"client.html#updated-client-client_updated"},{"type":"extras","title":"Joined lobby - `:joined_lobby` - Client messages","doc":"Sent whenever the client is added to a lobby; if you are subscribed to this topic you should also received a `client_updated` message. This separate message is to make matching on changing state easier.\n\n- `:lobby_id` - The ID of the lobby joined\n- `:user_id` - The ID of the user (which should also be present in the topic)\n\n```elixir\n%{\n  event: :joined_lobby,\n  lobby_id: Lobby.id(),\n  user_id: User.id()\n}\n```\n\n#","ref":"client.html#joined-lobby-joined_lobby"},{"type":"extras","title":"Left lobby - `:left_lobby` - Client messages","doc":"Sent whenever the client leaves a lobby; if you are subscribed to this topic you should also received a `client_updated` message. This separate message is to make matching on changing state easier.\n\n- `:lobby_id` - The ID of the lobby left\n- `:user_id` - The ID of the user (which should also be present in the topic)\n\n```elixir\n%{\n  event: :left_lobby,\n  lobby_id: Lobby.id(),\n  user_id: User.id()\n}\n```\n\n#","ref":"client.html#left-lobby-left_lobby"},{"type":"extras","title":"Disconnected client - `:client_disconnected` - Client messages","doc":"Sent when the client has no more connections\n\n- `:client` - A `Teiserver.Connections.Client` of the new client values\n\n```elixir\n%{\n  event: :client_disconnected,\n  client: Client.t()\n}\n```\n\n#","ref":"client.html#disconnected-client-client_disconnected"},{"type":"extras","title":"Destroyed client - `:client_destroyed` - Client messages","doc":"Sent when the client server process is destroyed.\n\n- `:user_id` - `Teiserver.Connections.User.id()` of the new client values\n\n```elixir\n%{\n  event: :client_destroyed,\n  user_id: User.id()\n}\n```","ref":"client.html#destroyed-client-client_destroyed"},{"type":"extras","title":"Lobby messages","doc":"# Lobby messages\nPubSub messages sent out in relation to changes to lobbies.\n\n## `Teiserver.Game.GlobalLobby`\nMessages for everybody for discoverability of lobbies.\n\n#","ref":"match.html"},{"type":"extras","title":"Open - `:lobby_opened` - Lobby messages","doc":"#","ref":"match.html#open-lobby_opened"},{"type":"extras","title":"Closed - `:lobby_closed` - Lobby messages","doc":"- `:lobby_id` - The id of the lobby closed\n\n```elixir\n%{\n  event: :lobby_closed,\n  lobby_id: Lobby.id()\n}\n```\n\n## `Teiserver.Game.Lobby:{lobby_id}`\nMessages relating to updates to the lobby in question.\n\n#","ref":"match.html#closed-lobby_closed"},{"type":"extras","title":"User joined - `:lobby_user_joined` - Lobby messages","doc":"Fired off whenever a user is added to the lobby member list.\n\n- `:client` - A `Teiserver.Connections.Client` of the client\n- `:shared_secret` - A string used to identify clients to the game host (since the in-game connection will be separate from the server). Though this value is broadcast to all subscribers it should only be forwarded to the host and relevant user.\n\n\n```elixir\n%{\n  event: :lobby_user_joined,\n  lobby_id: Lobby.id(),\n  client: Client.t(),\n  shared_secret: String.t()\n}\n```\n\n#","ref":"match.html#user-joined-lobby_user_joined"},{"type":"extras","title":"User left - `:lobby_user_left` - Lobby messages","doc":"Indicating a user has left the lobby\n\n- `:user_id` - The user_id of the leaver\n\n```elixir\n%{\n  event: :lobby_user_left,\n  lobby_id: Lobby.id(),\n  user_id: User.id()\n}\n```\n\n#","ref":"match.html#user-left-lobby_user_left"},{"type":"extras","title":"Message received - `:message_received` - Lobby messages","doc":"A message has been sent in the match chat.\n```elixir\n%{\n  event: :message_received,\n  lobby_id: Lobby.id(),\n  match_message: MatchMessage.t()\n}\n```\n\n#","ref":"match.html#message-received-message_received"},{"type":"extras","title":"Client state change - `:lobby_client_change` - Lobby messages","doc":"Note this will be sent in addition to normal client updated messages but by doing this we prevent people having to subscribe/unsubscribe from client update messages.\n\n- `:update_id` - An incremental number to allow for out of order messages\n- `:changes` - A map of the changes to the client\n\n```elixir\n%{\n  event: :client_updated,\n  update_id: integer(),\n  user_id: User.id(),\n  changes: map()\n}\n```\n\n#","ref":"match.html#client-state-change-lobby_client_change"},{"type":"extras","title":"State update - `:lobby_updated` - Lobby messages","doc":"- `:update_id` - An incremental number to allow for out of order messages\n- `:changes` - A map of the changes to the lobby\n\n```elixir\n%{\n  event: :lobby_updated,\n  lobby_id: Lobby.id(),\n  update_id: integer(),\n  changes: map()\n}\n```\n\n#","ref":"match.html#state-update-lobby_updated"},{"type":"extras","title":"Closed - `:lobby_closed` - Lobby messages","doc":"- `:lobby_id` - The id of the lobby closed\n\n```elixir\n%{\n  event: :lobby_closed,\n  lobby_id: Lobby.id()\n}\n```\n\n## `Teiserver.Game.LobbyHost:{lobby_id}`\nMessages specifically for hosting of the lobby. Separate from client related ones to allow for observability and greater control for users of the library.\n\n#","ref":"match.html#closed-lobby_closed"},{"type":"extras","title":"User requests to join - `:join_request` - Lobby messages","doc":"","ref":"match.html#user-requests-to-join-join_request"},{"type":"extras","title":"User messages","doc":"# User messages\nMessages for the account holder or admins regarding this user.\n\n`Teiserver.Account.User:{user_id}`\nStuff will probably happen here","ref":"user.html"},{"type":"extras","title":"Communication messages","doc":"# Communication messages\nMessages related to communication between players including RoomMessages, DirectMessages and LobbyMessages.\n\n## `Teiserver.Communication.Room:{room_id}`\nPubsub messages related to the Room.\n\n#","ref":"communication.html"},{"type":"extras","title":"Message received - Communication messages","doc":"Sent whenever a new message is sent into a room\n\n- `:message` - A `Teiserver.Communication.RoomMessage` representing the message received\n\n```elixir\n%{\n  event: :message_received,\n  message: RoomMessage.t()\n}\n```\n\n## `Teiserver.Communication.User:{user_id}`\nMessages related to the user and not a wider context.\n\n#","ref":"communication.html#message-received"},{"type":"extras","title":"Message received - Communication messages","doc":"Sent whenever a new DirectMessage is sent to this user.\n\n- `:message` - A `Teiserver.Communication.DirectMessage` representing the message received\n\n```elixir\n%{\n  event: :message_received,\n  message: DirectMessage.t()\n}\n```\n\n#","ref":"communication.html#message-received"},{"type":"extras","title":"Message sent - Communication messages","doc":"Sent whenever a new DirectMessage is sent by this user.\n\n- `:message` - A `Teiserver.Communication.DirectMessage` representing the message sent\n\n```elixir\n%{\n  event: :message_sent,\n  message: DirectMessage.t()\n}\n```\n\n## `Teiserver.Communication.Match:{match_id}`\nPubsub messages related to the Match.\n\n#","ref":"communication.html#message-sent"},{"type":"extras","title":"Message received - Communication messages","doc":"Sent whenever a new message is sent into a match\n\n- `:message` - A `Teiserver.Communication.MatchMessage` representing the message received\n\n```elixir\n%{\n  event: :message_received,\n  message: MatchMessage.t()\n}\n```","ref":"communication.html#message-received"}],"content_type":"text/markdown"}